!! ----------------------------------------------------------------------------
!! *PROGRAM* ReadInpNCDF_sounding_data
!! @version sacr 1.1
!!
!! *SRC_FILE*
!! sacr/ReadInpNCDF_sounding_data.f90
!!
!!
!! *LAST CHANGES*
!!
!!  -May 16, 2014: A.T. The following quality control parameters are introduced
!!                      when processing input set from sounding data:qc_wspd,
!!                      qc_deg,qc_tdry,qc_pres,qc_rh and valid_min and valid max 
!!                      for wind speed and direction, temperature, pressure and 
!!                      relative humidity
!!
!!  -May 15, 2014: A.T. Added new dimension in order to enable the use of the same
!!                      data structure for both sounding and merged sonde data
!!
!!
!!
!! *DESCRIPTION* 
!!
!! This program contains subroutines needed for reading the sounding netcdf data files
!! from ARM arhive.
!!
!!
!!  Copyright (C) 2014 McGill Clouds Research Group (http://www.clouds.mcgill.ca/)
!!  Contact email address: pavlos.kollias@mcgill.ca, aleksandra.tatarevic@mcgill.ca, iengjo@meteo.mcgill.ca
!!
!!  This source code is free software: you can redistribute it and/or modify
!!  it under the terms of the GNU Lesser General Public License as published by
!!  the Free Software Foundation, either version 3 of the License, or
!!  any later version.
!!
!!  This program is distributed in the hope that it will be useful,
!!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!!  GNU Lesser General Public License for more details.
!!
!!  You should have received a copy of the GNU Lesser General Public License
!!  along with this program.  If not, see <http://www.gnu.org/licenses/>.
!!
!! ----------------------------------------------------------------------------
!!
subroutine ReadInpNCDF_dim_sounding_data(InpFile,sounding,status)
!
Use netcdf
Use typeSizes
Use write_messages
Use radar_mods
!
Implicit None
!
Character(len=*),Intent(in)                              :: InpFile
Type(sacr_sounding_data),Intent(InOut)                   :: sounding
Integer,Intent(out)                                      :: status
!
Integer                                                  :: ncid, iDim,length
Integer                                                  :: nDims,nVars
Character(len=nf90_max_name)                             :: name, err_msg
Integer, Allocatable, Dimension(:)                       :: dim_lengths
Character(len=nf90_max_name), Allocatable , Dimension(:) :: dim_names
Integer                                                  :: aa
!
! define number of vertical profiles in sounding file
sounding%nt=1
!
!Open the file
status=  nf90_open(Trim(InpFile), NF90_NOWRITE, ncid)
If (status/=0) Then
err_msg = 'Error in my_nf90_open'
Goto 999
End If
!
!
! Get info on nDims and nVars
status= nf90_inquire(ncid, nDims, nVars)
If (status/=0) Then
err_msg = 'Error in my_nf90_inquire'
Goto 999
End If
!
Allocate(dim_names(1:nDims))
Allocate(dim_lengths(1:nDims))
!
! Get info on the dimensions
! Loop over the number of dimensions
!   
Do iDim = 1,nDims
!
status=nf90_inquire_dimension(ncid, idim, name=name, len=length)
!
If (status/=0) Then
err_msg = 'Error in my_nf90_inquire_dimension'
Goto 999
End If
!
dim_names(iDim)   = name
dim_lengths(iDim) = length
!
Select Case (name)
Case ('time')  ;  sounding%sounde   = length

   Case default
   status = 1
   err_msg = 'Unrecognised dimension: '
   Goto 999
End Select
enddo   ! iDim
!
999 If (status.Ne.0) Then
Call write_error(err_msg)
Else
status=  nf90_close(ncid)
Endif
!
return
end subroutine ReadInpNCDF_dim_sounding_data
!
subroutine ReadInpNCDF_var_sounding_data(InpFile,sounding,status)
!
Use netcdf
Use typeSizes
Use write_messages
Use radar_mods
!
Implicit None
!
Character(len=*),Intent(in)                              :: InpFile
Type(sacr_sounding_data),Intent(InOut)                   :: sounding
Integer,Intent(out)                                      :: status
!
integer                                                  :: nDims, nVars
integer                                                  :: ncid, iVar
Character(len=nf90_max_name), Allocatable , Dimension(:) :: var_names
Character(len=nf90_max_name)                             :: name, err_msg
!
real*4,Dimension(:),Allocatable    :: wspd,deg,tdry,pres,rh,alt
real*4                             :: wspd_valid_min,wspd_valid_max,deg_valid_min,deg_valid_max
real*4                             :: tdry_valid_min,tdry_valid_max,pres_valid_min, pres_valid_max
real*4                             :: rh_valid_min,rh_valid_max
integer*4,Dimension(:),Allocatable :: qc_wspd,qc_deg,qc_tdry,qc_pres,qc_rh
integer                            :: ns
integer                            :: i
Character(len=3)                   :: rlocation
real*8,parameter                   :: m999=-999.d0
!
!
ns=sounding%sounde
!
Allocate(wspd(ns),deg(ns),tdry(ns),pres(ns),rh(ns),alt(ns))
Allocate(qc_wspd(ns),qc_deg(ns),qc_tdry(ns),qc_pres(ns),qc_rh(ns))
!
wspd=real(m999); deg=real(m999); tdry=real(m999); pres=real(m999); rh=real(m999); alt=real(m999)
qc_wspd=int(m999); qc_deg=int(m999); qc_tdry=int(m999); qc_pres=int(m999); qc_rh=int(m999)
!
!Open the file
status=  nf90_open(Trim(InpFile), NF90_NOWRITE, ncid)
If (status/=0) Then
err_msg = 'Error in my_nf90_open'
Goto 999
End If
!
! Get info on nDims and nVars
status= nf90_inquire(ncid, nDims, nVars)
If (status/=0) Then
err_msg = 'Error in my_nf90_inquire'
Goto 999
End If
!
! Create arrays that contain the variables
Allocate(var_names(1:nVars))
!
Do  iVar=1,nVars
!
status= nf90_inquire_variable(ncid, iVar, name=name)
If (status/=0) Then
err_msg = 'Error in my_nf90_inquire_variable: '//name
Goto 999
Endif
!
!
Select Case (Trim(Adjustl(name)))
!
Case ('wspd')
status=nf90_get_var(ncid,iVar,wspd)
status = nf90_get_att(ncid, iVar, "valid_min", wspd_valid_min)
status = nf90_get_att(ncid, iVar, "valid_max", wspd_valid_max)
If (status/=0) Then
err_msg = 'Error in my_nf90_get_var: wspd'
Goto 999
End If
!
Case ('qc_wspd')
status=nf90_get_var(ncid,iVar,qc_wspd)
If (status/=0) Then
err_msg = 'Error in my_nf90_get_var: qc_wspd'
Goto 999
EndIf
!
Case ('deg')
status=nf90_get_var(ncid,iVar,deg)
status = nf90_get_att(ncid, iVar, "valid_min", deg_valid_min)
status = nf90_get_att(ncid, iVar, "valid_max", deg_valid_max)
If (status/=0) Then
err_msg = 'Error in my_nf90_get_var: deg'
Goto 999
End If
!
Case ('qc_deg')
status=nf90_get_var(ncid,iVar,qc_deg)
If (status/=0) Then
err_msg = 'Error in my_nf90_get_var: qc_deg'
Goto 999
EndIf
!
Case ('tdry')
status=nf90_get_var(ncid,iVar,tdry)
status = nf90_get_att(ncid, iVar, "valid_min", tdry_valid_min)
status = nf90_get_att(ncid, iVar, "valid_max", tdry_valid_max)
If (status/=0) Then
err_msg = 'Error in my_nf90_get_var: tdry'
Goto 999
End If
!
Case ('qc_tdry')
status=nf90_get_var(ncid,iVar,qc_tdry)
If (status/=0) Then
err_msg = 'Error in my_nf90_get_var: qc_tdry'
Goto 999
End If
!
Case ('pres')
status=nf90_get_var(ncid,iVar,pres)
status = nf90_get_att(ncid, iVar, "valid_min", pres_valid_min)
status = nf90_get_att(ncid, iVar, "valid_max", pres_valid_max)
If (status/=0) Then
err_msg = 'Error in my_nf90_get_var: pres'
Goto 999
End If
!
Case ('qc_pres')
status=nf90_get_var(ncid,iVar,qc_pres)
If (status/=0) Then
err_msg = 'Error in my_nf90_get_var: qc_pres'
Goto 999
End If
!
Case ('rh')
status=nf90_get_var(ncid,iVar,rh)
status = nf90_get_att(ncid, iVar, "valid_min", rh_valid_min)
status = nf90_get_att(ncid, iVar, "valid_max", rh_valid_max)
If (status/=0) Then
err_msg = 'Error in my_nf90_get_var: rh'
Goto 999
End If
!
Case ('qc_rh')
status=nf90_get_var(ncid,iVar,qc_rh)
If (status/=0) Then
err_msg = 'Error in my_nf90_get_var: qc_rh'
Goto 999
End If
!
Case ('alt')
status=nf90_get_var(ncid,iVar,alt)
If (status/=0) Then
err_msg = 'Error in my_nf90_get_var: alt'
Goto 999
End If
!
End Select
!
If (status/=0) Then
err_msg = 'Error in my_nf90_get_var:'
Goto 999
End If
!
End Do
!
999 If (status.Ne.0) Then
Call write_error(err_msg)
Else
status=nf90_close(ncid)
Endif
!
!----------------------------------------------------------------------------------
!! input data managment
!! 
where(wspd < wspd_valid_min) wspd=-999.e0
where(wspd > wspd_valid_max) wspd=-999.e0
where(qc_wspd==0) sounding%wf=dble(wspd)
!!
where(deg < deg_valid_min) deg=-999.e0
where(deg > deg_valid_max) deg=-999.e0
where(qc_deg==0) sounding%wd=dble(deg)
!!
where(tdry < tdry_valid_min) tdry=-999.e0
where(tdry > tdry_valid_max) tdry=-999.e0
where(qc_tdry==0) sounding%Temperature=dble(tdry)
!!
where(pres < pres_valid_min) pres=-999.e0
where(pres > pres_valid_max) pres=-999.e0
where(qc_pres==0) sounding%pressure=dble(pres)
!!
where(rh < rh_valid_min) rh=-999.e0
where(rh> rh_valid_max) rh=-999.e0
where(qc_rh==0) sounding%relative_hum=dble(rh)
!!
sounding%altitude=dble(alt)
!!
!!
!! wind direction change 
!! in input arm files North is zero degrees. Convention here: East is zero degrees.
!
where(sounding%wd==m999) deg=real(m999)
!
do i=1,ns
if ((deg(i)>=0).and.(deg(i)<=90)) Then
sounding%wd(i)=90.d0-dble(deg(i))
else if ((deg(i)>90).and.(deg(i)<=360)) Then
sounding%wd(i)=360.d0-(dble(deg(i))-90.d0)
else
sounding%wd(i)=m999 
endif
enddo
!
!
Deallocate(wspd,deg,tdry,pres,rh,alt)
Deallocate(qc_wspd,qc_deg,qc_tdry,qc_pres,qc_rh)
!
return
end subroutine ReadInpNCDF_var_sounding_data


