!! ----------------------------------------------------------------------------
!! *PROGRAM* gasabs_module
!! @version sacr 1.0
!!
!! *SRC_FILE*
!! sacr/gasabs_module.f90
!!
!!
!! *LAST CHANGES*
!!
!!
!! *DESCRIPTION* 
!!
!! This program contains the subroutines for calculation of gasseous attenuation
!! by various gases using the  method of ROSENKRANZ (1998) 
!! -- reference "Water vapor microwave continuum absorption: 
!! a comparison of measurements and results"  (To appear in Radio Science)
!! it is provided by IGMK, and modified to be used in sacr
!!
!!
!!
!!  Copyright (C) 2014 McGill Clouds Research Group (http://www.clouds.mcgill.ca/)
!!  Contact email address: pavlos.kollias@mcgill.ca, aleksandra.tatarevic@mcgill.ca, iengjo@meteo.mcgill.ca
!!
!!  This source code is free software: you can redistribute it and/or modify
!!  it under the terms of the GNU Lesser General Public License as published by
!!  the Free Software Foundation, either version 3 of the License, or
!!  any later version.
!!
!!  This program is distributed in the hope that it will be useful,
!!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!!  GNU Lesser General Public License for more details.
!!
!!  You should have received a copy of the GNU Lesser General Public License
!!  along with this program.  If not, see <http://www.gnu.org/licenses/>.
!!
!! ----------------------------------------------------------------------------


module gasabs_module 
    !
    ! Description:
    !   This module contains functions, that calculate the absortpion for
    !   different gases. It is provided to McGill by IGMK.
    !
    ! History:
    !
    ! Version   Date       Comment
    ! -------   ----       -------
    ! 0.1       21/09/2009 Code adaption from G. Petty - M. Mech
    ! 0.2       27/02/2013 Application of European Standards for Writing and
    !                      Documenting Exchangeable Fortran 90 Code - M. Mech
    ! Code Description:
    !   Language:		Fortran 90.
    !   Software Standards: "European Standards for Writing and  
    !     Documenting Exchangeable Fortran 90 Code". 
    !
    !
    ! CHANGES:
    !    A.T.,McGill    18.09.2013  modified the header, interface and performed
    !                               systematic conversion to double precision 
    !                               (it was not done properly in the 0.2 version)
    ! 
    !******************************************************************
    !This file contains subroutines for computing atmospheric absorption at
    !microwave wavelengths, as supplied by P. Rosenkranz (2/98) via his
    !anonymous ftp server (mesa.mit.edu; login anonymous; go to phil/lpl_rt)AN
    !Consolidated into one file by G. Petty
    !
    !**********************************************************
    !  Begin summaries
    !**********************************************************
    !     real function absn2(tempK,pres,freq)
    !     absn2 = ABSORPTION COEFFICIENT DUE TO NITROGEN IN AIR
    !             (NEPER/KM)
    !     tempK = TEMPERATURE (K)
    !     pres = PRESSURE (MB)
    !     freq = FREQUENCY (GHZ)
    !
    !************************************************************************
    !     real function o2abs(tempK,pres,vapden,freq)
    !
    !     PURPOSE: RETURNS ABSORPTION COEFFICIENT DUE TO OXYGEN IN AIR,
    !              IN NEPERS/KM
    !
    !     NAME    UNITS    DESCRIPTION        VALID RANGE
    !
    !     tempK   KELVIN   TEMPERATURE        (UNCERTAIN)
    !     PRES   MILLIBARS PRESSURE           (3 TO 1000)
    !     VAPDEN  G/M**3   WATER VAPOR DENSITY  (ENTERS LINEWIDTH CALCULATION
    !                      DUE TO GREATER BROADENING EFFICIENCY OF H2O)
    !     FREQ    GHZ      FREQUENCY          (0 TO 900)
    !*************************************************************
    !     real function abh2o(tempK,pres,rho,freq)
    !
    ! PURPOSE- COMPUTE ABSORPTION COEF IN ATMOSPHERE DUE TO WATER VAPOR
    !
    !      NAME    UNITS    I/O  DESCRIPTON            VALID RANGE
    !      tempK   KELVIN    I   TEMPERATURE
    !      pres    MILLIBAR  I   PRESSURE              .1 TO 1000
    !      rho     G/M**3    I   WATER VAPOR DENSITY
    !      freq    GHZ       I   FREQUENCY             0 TO 800
    !      abh2o   NEPERS/KM O   ABSORPTION COEFFICIENT
    !
    !****************************************************************
    !     real function abliq(water,freq,tempK)
    !     COMPUTES ABSORPTION IN NEPERS/KM BY SUSPENDED WATER DROPLETS
    !     FROM EQUATIONS OF LIEBE, HUFFORD AND MANABE
    !     (INT. J. IR & MM WAVES V.12(17) JULY 1991
    !     WATER IN G/M**3
    !     FREQ IN GHZ     (VALID FROM 0 TO 1000 GHZ)
    !     tempK IN KELVIN
    !        PWR 8/3/92
    !
    !*************************************************************
contains 
    !*************************************************************
    !
    !  Begin actual function definitions
    !
    !**************************************************************
    function absn2(tempK,pres,freq)
        !     absn2 = ABSORPTION COEFFICIENT DUE TO NITROGEN IN AIR
        !             (NEPER/KM)
        !     tempK = TEMPERATURE (K)
        !     pres = PRESSURE (MB)
        !     freq = FREQUENCY (GHZ)
        !
        implicit none

        real*8, intent(in) :: tempK,pres,freq
        real*8 :: absn2
        real*8 :: th

        th = 300.d0/tempK
        absn2 = 6.4d-14*pres*pres*freq*freq*th**3.55d0

        return

    end function absn2
    !
    !
    function o2abs(tempK,pres,vapden,freq)
        !
        !     PURPOSE: RETURNS ABSORPTION COEFFICIENT DUE TO OXYGEN IN AIR,
        !              IN NEPERS/KM
        !
        !      5/1/95  P. Rosenkranz
        !
        !     ARGUMENTS:
        !
        !     NAME    UNITS    DESCRIPTION        VALID RANGE
        !
        !     tempK   KELVIN   TEMPERATURE        (UNCERTAIN)
        !     pres    MILLIBARS PRESSURE           (3 TO 1000)
        !     vapden  G/M**3   WATER VAPOR DENSITY  (ENTERS LINEWIDTH CALCULATION
        !                      DUE TO GREATER BROADENING EFFICIENCY OF H2O)
        !     freq    GHZ      FREQUENCY          (0 TO 900)
        !
        !     REFERENCE FOR EQUATIONS AND COEFFICIENTS:
        !     P.W. ROSENKRANZ, CHAP. 2 AND APPENDIX, IN ATMOSPHERIC REMOTE SENSING
        !      BY MICROWAVE RADIOMETRY (M.A. JANSSEN, ED. 1993)
        !     AND H.J. LIEBE ET AL, JQSRT V.48, PP.629-643 (1992)
        !     (EXCEPT: SUBMILLIMETER LINE INTENSITIES FROM HITRAN92)
        !
        implicit none
        real*8             :: pi  
        integer            :: k

        real*8, intent(in) :: tempK,pres,vapden,freq
        real :: th, th1, b, preswv, presda, den, dfnr, df
        real :: str, sf1, sf2, loc_sum, y

        real*8 :: o2abs
        !      WIDTHS IN MHZ/MB
        real*8 :: x = 0.8d0, &
                          wb300 = 0.56d0
        real*8, dimension(40) :: w300 = (/&
          1.63d0, 1.646d0, 1.468d0, 1.449d0, 1.382d0, 1.360d0,&
          1.319d0, 1.297d0, 1.266d0, 1.248d0, 1.221d0, 1.207d0, 1.181d0, 1.171d0,&
          1.144d0, 1.139d0, 1.110d0, 1.108d0, 1.079d0, 1.078d0, 1.05d0, 1.05d0,&
          1.02d0, 1.02d0, 1.0d0, 1.0d0, 0.97d0, 0.97d0, 0.94d0, 0.94d0, 0.92d0, 0.92d0, &
          0.89d0, 0.89d0, 1.92d0, 1.92d0, 1.92d0, 1.81d0, 1.81d0, 1.81d0/)
        real*8, dimension(40) :: y300 = (/&
          -0.0233d0,  0.2408d0, -0.3486d0,  0.5227d0,&
          -0.5430d0,  0.5877d0, -0.3970d0,  0.3237d0, -0.1348d0,  0.0311d0,&
          0.0725d0, -0.1663d0,  0.2832d0, -0.3629d0,  0.3970d0, -0.4599d0,&
          0.4695d0, -0.5199d0,  0.5187d0, -0.5597d0,  0.5903d0, -0.6246d0,&
          0.6656d0, -0.6942d0,  0.7086d0, -0.7325d0,  0.7348d0, -0.7546d0,&
          0.7702d0, -0.7864d0,  0.8083d0, -0.8210d0,  0.8439d0, -0.8529d0, &
          0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0/)
        real*8, dimension(40) :: v = (/&
          0.0079d0, -0.0978d0,  0.0844d0, -0.1273d0,&
          0.0699d0, -0.0776d0,  0.2309d0, -0.2825d0,  0.0436d0, -0.0584d0,&
          0.6056d0, -0.6619d0,  0.6451d0, -0.6759d0,  0.6547d0, -0.6675d0,&
          0.6135d0, -0.6139d0,  0.2952d0, -0.2895d0,  0.2654d0, -0.2590d0,&
          0.3750d0, -0.3680d0,  0.5085d0, -0.5002d0,  0.6206d0, -0.6091d0,&
          0.6526d0, -0.6393d0,  0.6640d0, -0.6475d0,  0.6729d0, -0.6545d0,&
          0.d0, 0.d0, 0.d0, 0.d0, 0.d0, 0.d0/)
        !      LINES ARE ARRANGED 1-,1+,3-,3+,ETC. IN SPIN-ROTATION SPECTRUM
        real*8, dimension(40) :: f = (/&
          118.7503d0, 56.2648d0, 62.4863d0, 58.4466d0, 60.3061d0, 59.5910d0,&
          59.1642d0, 60.4348d0, 58.3239d0, 61.1506d0, 57.6125d0, 61.8002d0,&
          56.9682d0, 62.4112d0, 56.3634d0, 62.9980d0, 55.7838d0, 63.5685d0,&
          55.2214d0, 64.1278d0, 54.6712d0, 64.6789d0, 54.1300d0, 65.2241d0,&
          53.5957d0, 65.7648d0, 53.0669d0, 66.3021d0, 52.5424d0, 66.8368d0,&
          52.0214d0, 67.3696d0, 51.5034d0, 67.9009d0, 368.4984d0, 424.7631d0,&
          487.2494d0, 715.3932d0, 773.8397d0, 834.1453d0/)
        real*8, dimension(40) :: s300 = (/&
          0.2936d-14, 0.8079d-15, 0.2480d-14, 0.2228d-14,&
          0.3351d-14, 0.3292d-14, 0.3721d-14, 0.3891d-14,&
          0.3640d-14, 0.4005d-14, 0.3227d-14, 0.3715d-14,&
          0.2627d-14, 0.3156d-14, 0.1982d-14, 0.2477d-14,&
          0.1391d-14, 0.1808d-14, 0.9124d-15, 0.1230d-14,&
          0.5603d-15, 0.7842d-15, 0.3228d-15, 0.4689d-15,&
          0.1748d-15, 0.2632d-15, 0.8898d-16, 0.1389d-15,&
          0.4264d-16, 0.6899d-16, 0.1924d-16, 0.3229d-16,&
          0.8191d-17, 0.1423d-16, 0.6460d-15, 0.7047d-14,&
          0.3011d-14, 0.1826d-14, 0.1152d-13, 0.3971d-14/)
        real*8, dimension(40) :: be = (/&
          0.009d0, 0.015d0, 0.083d0, 0.084d0, 0.212d0, 0.212d0, 0.391d0, 0.391d0, 0.626d0, 0.626d0,&
          0.915d0, 0.915d0, 1.26d0, 1.26d0, 1.66d0, 1.665d0, 2.119d0, 2.115d0, 2.624d0, 2.625d0,&
          3.194d0, 3.194d0, 3.814d0, 3.814d0, 4.484d0, 4.484d0, 5.224d0, 5.224d0, 6.004d0, 6.004d0, &
          6.844d0, 6.844d0,7.744d0, 7.744d0, 0.048d0, 0.044d0, 0.049d0, 0.145d0, 0.141d0, 0.145d0/)
 
        pi=dasin(1.0d+0)*2.d0
        th = 300.d0/tempK
        th1 = th-1.d0
        b = th**x
        preswv = vapden*tempK/217.d0
        presda = pres-preswv
        den = 0.001d0*(presda*b + 1.1d0*preswv*th)
        dfnr = wb300*den
        loc_sum = 1.6d-17*freq*freq*dfnr/(th*(freq*freq + dfnr*dfnr))
        do k=1,40
            df = w300(k)*den
            y = 0.001d0*pres*b*(y300(k)+v(k)*th1)
            str = s300(k)*dexp(-be(k)*th1)
            sf1 = (df + (freq-f(k))*y)/((freq-f(k))**2 + df*df)
            sf2 = (df - (freq+f(k))*y)/((freq+f(k))**2 + df*df)
            loc_sum = loc_sum + str*(sf1+sf2)*(freq/f(k))**2
        end do
        o2abs = 0.5034d12*loc_sum*presda*th**3/pi

        return

    end function o2abs

    !*************************************************************
    function abh2o(tempK,pres,rho,freq)
        !
        !  NAME- ABH2O    LANGUAGE- FORTRAN 77
        !
        ! PURPOSE- COMPUTE ABSORPTION COEF IN ATMOSPHERE DUE TO WATER VAPOR
        !
        !  CALLING SEQUENCE PARAMETERS-
        !    SPECIFICATIONS
        !      NAME    UNITS    I/O  DESCRIPTON            VALID RANGE
        !      tempK   KELVIN    I   TEMPERATURE
        !      pres    MILLIBAR  I   PRESSURE              .1 TO 1000
        !      rho     G/M**3    I   WATER VAPOR DENSITY
        !      freq    GHZ       I   FREQUENCY             0 TO 800
        !      abh2o   NEPERS/KM O   ABSORPTION COEFFICIENT
        !
        !   REFERENCES-
        !   LINE INTENSITIES FROM HITRAN92 (SELECTION THRESHOLD=
        !     HALF OF CONTINUUM ABSORPTION AT 1000 MB).
        !   WIDTHS MEASURED AT 22,183,380 GHZ, OTHERS CALCULATED:
        !     H.J.LIEBE AND T.A.DILLON, J.CHEM.PHYS. V.50, PP.727-732 (1969) &
        !     H.J.LIEBE ET AL., JQSRT V.9, PP. 31-47 (1969)  (22GHz);
        !     A.BAUER ET AL., JQSRT V.37, PP.531-539 (1987) &
        !     ASA WORKSHOP (SEPT. 1989) (380GHz);
        !     AND A.BAUER ET AL., JQSRT V.41, PP.49-54 (1989) (OTHER LINES).
        !   AIR-BROADENED CONTINUUM BASED ON LIEBE & LAYTON, NTIA
        !     REPORT 87-224 (1987); SELF-BROADENED CONTINUUM BASED ON
        !     LIEBE ET AL, AGARD CONF. PROC. 542 (MAY 1993),
        !     BUT READJUSTED FOR LINE SHAPE OF
        !     CLOUGH et al, ATMOS. RESEARCH V.23, PP.229-241 (1989).
        !
        !   REVISION HISTORY-
        !    DATE- OCT.6, 1988  P.W.ROSENKRANZ - EQS AS PUBL. IN 1993.
        !          OCT.4, 1995  PWR- USE CLOUGH'S DEFINITION OF LOCAL LINE
        !                   CONTRIBUTION,  HITRAN INTENSITIES, ADD 7 LINES.
        !          OCT. 24, 95  PWR -ADD 1 LINE.
        !          JULY 7, 97   PWR -SEPARATE COEFF. FOR SELF-BROADENING,
        !                       REVISED CONTINUUM.
        !    DATE  SEP.3, 2012  SELF- AND FOREIGN-BROADENING PARAMETER FOR H2O LINE AT 22.235 GHz
        !                           CHANGED ACCORDING TO Liljegren et al.,2005, IEEE TGaRS
        !                       SELF- AND FOREIGN-CONTINUUM CONTRIBUTION CHANGED
        !                           ACCORDING TO Turner et al., 2009, IEEE TGaRS
        !
        !   LOCAL VARIABLES:

        implicit none

        real*8, intent(in) :: tempK,pres,rho,freq
        real*8 :: abh2o
        integer, parameter :: nlines = 15
        integer :: i,j
        real*8  :: df(2)
        real*8 :: pvap,pda,den,ti,ti2,loc_sum,width,wsq,s,base,res,con
        !     LINE FREQUENCIES:
        real*8, dimension(nlines) :: fl = (/&
          22.2351d0, 183.3101d0, 321.2256d0, 325.1529d0, 380.1974d0, 439.1508d0,&
          443.0183d0, 448.0011d0, 470.8890d0, 474.6891d0, 488.4911d0, 556.9360d0,&
          620.7008d0, 752.0332d0, 916.1712d0/)
        !     LINE INTENSITIES AT 300K:
        real*8, dimension(nlines) :: s1 = (/&
          0.1310d-13, 0.2273d-11, 0.8036d-13, 0.2694d-11, 0.2438d-10,&
          0.2179d-11, 0.4624d-12, 0.2562d-10, 0.8369d-12, 0.3263d-11,&
          0.6659d-12, 0.1531d-08, 0.1707d-10, 0.1011d-08, 0.4227d-10/)
        !     T COEFF. OF INTENSITIES:
        real*8, dimension(nlines) :: b2 = (/&
          2.144d0, 0.668d0, 6.179d0, 1.541d0, 1.048d0, 3.595d0, 5.048d0, 1.405d0,&
          3.597d0, 2.379d0, 2.852d0, 0.159d0, 2.391d0, 0.396d0, 1.441d0/)
        !     AIR-BROADENED WIDTH PARAMETERS AT 300K:
        real*8, dimension(nlines) :: w3 = (/&
          0.002656d0, 0.00281d0, 0.0023d0, 0.00278d0, 0.00287d0, 0.0021d0, 0.00186d0,&
          0.00263d0, 0.00215d0, 0.00236d0, 0.0026d0, 0.00321d0, 0.00244d0, 0.00306d0, 0.00267d0/)
        !     T-EXPONENT OF AIR-BROADENING:
        real*8, dimension(nlines) :: x = (/&
          0.69d0, 0.64d0, 0.67d0, 0.68d0, 0.54d0, 0.63d0, 0.60d0, 0.66d0, 0.66d0,&
          0.65d0, 0.69d0, 0.69d0, 0.71d0, 0.68d0, 0.70d0/)
        !     SELF-BROADENED WIDTH PARAMETERS AT 300K:
        real*8, dimension(nlines) :: ws = (/&
          0.0127488d0, 0.01491d0, 0.0108d0, 0.0135d0, 0.01541d0, 0.0090d0, 0.00788d0,&
          0.01275d0, 0.00983d0, 0.01095d0, 0.01313d0, 0.01320d0, 0.01140d0, 0.01253d0, 0.01275d0/)
        !     T-EXPONENT OF SELF-BROADENING:
        real*8, dimension(nlines) :: xs = (/&
          0.61d0, 0.85d0, 0.54d0, 0.74d0, 0.89d0, 0.52d0, 0.50d0, 0.67d0, 0.65d0, 0.64d0, 0.72d0,&
          1.0d0, 0.68d0, 0.84d0, 0.78d0/)

        if(rho <= 0.d0) then
            abh2o = 0.d0
            return
        endif
        pvap = rho * tempK / 217.d0
        pda = pres - pvap
        den = 3.335d16 * rho
        ti = 300.d0 / tempK
        ti2 = ti**2.5d0
        !
        !      continuum terms
        con = (5.43d-10*1.105d0*pda*ti**3 + 1.8d-8*0.79d0*pvap*ti**7.5d0)*pvap*freq*freq
        !
        !      add resonances
        loc_sum = 0.d0
        do i=1,nlines
            width = w3(i)*pda*ti**x(i) + ws(i)*pvap*ti**xs(i)
            wsq = width*width
            s = s1(i)*ti2*dexp(b2(i)*(1.d0-ti))
            df(1) = freq - fl(i)
            df(2) = freq + fl(i)
            !  use clough's definition of local line contribution
            base = width/(562500.d0 + wsq)
            !  do for positive and negative resonances
            res = 0.d0
            do j=1,2
                if(abs(df(j)).lt.750.d0) res = res + width/(df(j)**2+wsq) - base
            end do
            loc_sum = loc_sum + s*res*(freq/fl(i))**2
        end do
        abh2o = 0.3183d-4*den*loc_sum + con

        return

    end function abh2o
    !*************************************************************
    function abliq(water,freq,tempK)
        !     COMPUTES ABSORPTION IN NEPERS/KM BY SUSPENDED WATER DROPLETS
        !     FROM EQUATIONS OF LIEBE, HUFFORD AND MANABE
        !     (INT. J. IR & MM WAVES V.12(17) JULY 1991
        !     water IN G/M**3
        !     freq  IN GHZ     (VALID FROM 0 TO 1000 GHZ)
        !     tempK IN KELVIN
        !        PWR 8/3/92
        !
        implicit none

        real*8 :: water, freq, tempK
        real*8 :: abliq
        real*8 :: theta1, eps0, eps1, eps2, fp, fs
        double complex :: eps,re

        if(water <= 0.d0) then
            abliq = 0.d0
            return
        endif
        theta1 = 1.d0-300.d0/tempK
        eps0 = 77.66d0 - 103.3d0*theta1
        eps1 = 0.0671d0*eps0
        eps2 = 3.52d0 + 7.52d0*theta1
        fp = (316.d0*theta1 + 146.4d0)*theta1 +20.20d0
        fs = 39.8d0*fp
        eps = (eps0-eps1)/cmplx(1.d0,freq/fp) +&
          (eps1-eps2)/cmplx(1.d0,freq/fs) +eps2
        re = (eps-1.d0)/(eps+2.d0)
        abliq = -0.06286d0*aimag(re)*freq*water

        return

    end function abliq

end module gasabs_module 
 
!- End of module header
