!! ----------------------------------------------------------------------------
!! *PROGRAM* processing
!! @version sacr 1.1
!!
!! *SRC_FILE*
!! sacr/processing.f90
!!
!!
!!
!! *LAST CHANGES*
!!
!!  -May 26, 2014: A.T. Subroutine dealiasing:
!!                        - Saved computations of wind speed and directions in radar coordinates
!!                          
!!        
!!  -May 23, 2014: A.T. Subroutine EstimateNoiseFloor_yppi:
!!                         - Corrected bug with indices in EstimateNoiseFloor_yppi-condition
!!                             old : if (MaxVal(raw_data_per_scan%DBZ_rc(igate,:))/=m999) Then
!!                             new : if (MaxVal(raw_data_per_scan%DBZ_rc(:,itime))/=m999) Then
!!
!!
!!  -May 20, 2014: A.T. Subroutine CheckElFlippingAndRepeating :
!!                         - Criterium for checking if elevations increase or decrease in time
!!                            modified -Instead of checking the first and last 50 elevation values,
!!                            the values +/- 20 from the middle of the scan are examined.
!!                         - Corrected mismatch in one elevation value when estimating the ending index in
!!                            increasing elevation array
!!
!!  -May 16, 2014: A.T. Subroutine GetSoundingID :
!!                         - Included computation of variable dt_sounding_scan (time interval
!!                           in seconds  between the sacr scan time  and the closest sounding) as an output.
!!
!!  -May 14, 2014: A.T.  Subroutine InsectRemoval:
!!                         - Icluded possibility to determine the height to which the LDR threshold for
!!                           insects is applied in the case when only ceilometer or only sounding
!!                           (sounde, merged sounde) data are available as an input.
!!
!!
!!
!! *DESCRIPTION* 
!!
!!  This program  contains the subroutines needed for post-processing scanning radar data from ARM archive:
!! -for the construction of hydrometeor mask and removal of not significant radar returns
!! -for the attenution correction for gasses using the nearest-in-time atmospheric sounding
!! -for the corrrection for the velocity folding that makes use of a first guess for the true Doppler velocity using 
!!    horizontal wind measurements from the nearest sounding
!! -for the insect removal 
!! -for  the removal of the second trip echo 
!!
!!
!!  Copyright (C) 2014 McGill Clouds Research Group (http://www.clouds.mcgill.ca/)
!!  Contact email address: pavlos.kollias@mcgill.ca, aleksandra.tatarevic@mcgill.ca, iengjo@meteo.mcgill.ca
!!
!!  This source code is free software: you can redistribute it and/or modify
!!  it under the terms of the GNU Lesser General Public License as published by
!!  the Free Software Foundation, either version 3 of the License, or
!!  any later version.
!!
!!  This program is distributed in the hope that it will be useful,
!!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!!  GNU Lesser General Public License for more details.
!!
!!  You should have received a copy of the GNU Lesser General Public License
!!  along with this program.  If not, see <http://www.gnu.org/licenses/>.
!!
!! ----------------------------------------------------------------------------
!!
!!
subroutine ExtractDataPerScan(iscan,it_s,it_e,flipID,smode,raw_data,raw_data_per_scan)
use radar_mods
use write_messages
Implicit None
! 
Integer,Intent(In)                :: iscan ! scan number
Integer,Intent(In)                :: it_s,it_e ! first and last index of elevation array, from lowest to highest & repeated elevations exluded
Integer,Intent(In)                :: flipID    ! =0 raw elevations from lowest to highest
                                               ! =1 flipping is required as raw elevations go from highest to lowest
                                               ! =-1 disregard writting the output file as 1.raw elevations are identical, or
                                               !                                           2. span of valid elevations<20 degrees
Character(len=*),Intent(In)       :: smode     ! sacr_param%mode
Type(sacr_raw_data),Intent(In)                       :: raw_data
Type(sacr_raw_data_per_scan),Intent(InOut)           :: raw_data_per_scan    ! corrected for flipping and repeated elevations
!
Integer                                              :: nt,ng
Integer                                              :: i1,i2
Integer                                              :: it
Integer                                              :: ii
!
character(len=15)          :: time_str
integer*4                  :: t(9)
!
nt=raw_data_per_scan%ntimes
ng=raw_data_per_scan%ngates
!
if (flipID==0) Then  ! no flipping required
    i1=it_s
    i2=it_e
    ii=1
else if (flipID==1) Then ! flipping required
    i1=it_e
    i2=it_s
    ii=-1
else
    call write_error('subr. ExtractDataPerScan - flipID problem')
    call Exit (1) 
endif
!
raw_data_per_scan%elevation(1:nt)        = raw_data%elevation(i1:i2:ii)
raw_data_per_scan%azimuth(1:nt)          = raw_data%azimuth(i1:i2:ii)
raw_data_per_scan%time_s(1:nt)           = raw_data%time_s(i1:i2:ii)
raw_data_per_scan%DBZ(:,1:nt)            = raw_data%DBZ(:,i1:i2:ii)
raw_data_per_scan%VELO(:,1:nt)           = raw_data%VELO(:,i1:i2:ii)
raw_data_per_scan%WIDTH(:,1:nt)          = raw_data%WIDTH(:,i1:i2:ii)
raw_data_per_scan%LDR(:,1:nt)            = raw_data%LDR(:,i1:i2:ii)
raw_data_per_scan%SNR(:,1:nt)            = raw_data%SNR(:,i1:i2:ii)
!
! Range corrected reflectivity used later for masking
!
do it=1,nt
raw_data_per_scan%DBZ_rc(:,it)=raw_data_per_scan%DBZ(:,it) - 20.d0*dlog10(raw_data%range(:)*1.d-3)
enddo
!
! range 
raw_data_per_scan%range=raw_data%range
!
! Add fixed angle variable and time at the middle of the scan
it=idnint(0.5d0*dble(nt))
! fixed angle
If (Trim(smode)/='ppimo') Then ! fixed azimuth
raw_data_per_scan%fixed_angle(1)=idnint(raw_data_per_scan%azimuth(it))
Else ! fixed elevation
raw_data_per_scan%fixed_angle(1)=idnint(raw_data_per_scan%elevation(it))
EndIf
!
! time
call EpochTime2ReadableFormat(int(raw_data_per_scan%time_s(it),4),t)
time_str=''
write(time_str,'(a,(I4.4,I2.2,I2.2,"_",I2.2,I2.2,I2.2))') trim(time_str),t(6)+1900, t(5)+1, t(4), t(3), t(2), t(1)
raw_data_per_scan%time_str_index=it
raw_data_per_scan%time_str=Trim(time_str)
!
!
! radar_time
do it=1,nt
call EpochTime2ReadableFormat(int(raw_data_per_scan%time_s(it),4),t)
raw_data_per_scan%radar_time(it)=t(3)+t(2)/60.d0 + t(1)/3600.d0
enddo
!
return
end subroutine ExtractDataPerScan
!!
subroutine CheckElFlippingAndRepeating(smode,iscan,raw_data,ntimes,it_s,it_e,flipID)
use radar_mods
use write_messages
Implicit None
! 
Character(len=*),Intent(In)                         :: smode  ! sacr_param%mode
Integer,Intent(In)                                  :: iscan
Type(sacr_raw_data),Intent(In)                      :: raw_data
Integer,Intent(Out)                                 :: ntimes ! total number of elevatios corrected for repetition
Integer,Intent(Out)                                 :: it_s   ! the first index for elevations,corrected for repetition 
Integer,Intent(Out)                                 :: it_e   ! the last  index for elevations,corrected for repetition 
Integer,Intent(Out)                                 :: flipID ! if flipping is required flipID=1; not reqiered =0, all elevs are the same flipID=-1
!
!
integer                                              :: nt
real*8,Dimension(:),Allocatable                      :: elev
real*8,Dimension(:),Allocatable                      :: azimuth
real*8,parameter                                     :: m999=-999.d0
integer                                              :: it1, it2
integer                                              :: i1,i2,t1,t2
real*8                                               :: s1,s2
!
integer                                              :: j1,j2
integer                                              :: ij1,ij2,ij
integer                                              :: j_start,j_end
integer                                              :: istatus
integer,Dimension(:),Allocatable                     :: work
real*8                                               :: el_diff
integer                                              :: it
!
!
it1=raw_data%sweep_start_ray_index(iscan)
it2=raw_data%sweep_end_ray_index(iscan)
nt=it2-it1+1
!
! if everything is ok
! init
it_s=it1  
it_e=it2
ntimes=nt
flipID=0  ! assume 0->180 direction when scanning 
!
! check if direction is correct
Allocate(elev(nt))
Allocate(azimuth(nt))
elev(1:nt)=raw_data%elevation(it1:it2)
azimuth(1:nt)=raw_data%azimuth(it1:it2)

if (MinVal(elev)==MaxVal(elev)) flipID=-1 ! do nothing
if (MinVal(elev)>40) flipID=-1 ! do nothing
if ((MaxVal(elev)< 105) .and. (smode=='cwrhi')) flipID=-1 ! do nothing
if ((MaxVal(elev)< 105) .and. (smode=='hsrhi')) flipID=-1 ! do nothing
if ((MaxVal(elev)< 60 ) .and. (smode=='blrhi')) flipID=-1 ! do nothing

if (flipID==-1) go to 111


i1=Max(1,nint(0.5*real(nt)-20.)) ; i2=nint(0.5*real(nt))
t1=count(elev(i1:i2)/=m999)
s1=Sum(elev(i1:i2),MASK=elev(i1:i2)/=m999)/dble(t1)

i1=i2 ; i2=Min(i2+20,nt)
t2=count(elev(i1:i2)/=m999)
s2=Sum(elev(i1:i2),MASK=elev(i1:i2)/=m999)/dble(t2)
!
if (s1>s2) Then ! elevations are from 180 to 0; flipping is to be performed
flipID=1
! get elev and azimuth in increasing order as in for the case flipID=0
elev(1:nt)=elev(nt:1:-1) ! flip elev
azimuth(1:nt)=azimuth(nt:1:-1) ! flip azimuth
endif
!
!
! now all elev and azimuth are in increasing order
! and go from 1->nt
!
!
! check if it_s and it_e have to be changed due to repetition
! it_s corresponds to elevation around 0 if flipID=0  and elevation around 180 if flipID=1
! it_e corresponds to elevation around 180 if flipID=0  and elevation around 0 if flipID=1
!
!
! assume that there is no need for any correction 
! init
istatus=-1 ! no need for any correction
ij1=1 ; ij2=nt
! 
!  first check if elevations are always increasing 
A1:do it=ij1+1,ij2
if (elev(it)<=elev(it-1)) Then
istatus=1
Exit A1
endif
enddo A1
!
j1=-1; j2=-1


if (istatus==1) Then ! we have to correct indices, as elevations are in not increasing order
!
      Allocate(work(nt-1)) ! number of delta elevations 
      !
      work=-1 ! everywhere 
      do it=ij1+1,ij2
      if (elev(it)<=elev(it-1)) Then
      work(it-1)=0  ! 0  if the succeeding elvation is less than the previous one 
      else
      work(it-1)=1 ! increasing order of elevations
      endif
      enddo
!
      j_start=ij1 ; j_end=ij2-1
222 continue
      ! find j1
      A2:do it=j_start,j_end
            if (work(it)==1)then
               j1=it
               if (j1==j_end) goto 223
               exit A2
            endif
      enddo A2
      !
      ! find j2
      if (j1>=1) Then
      A3: do it=j1,j_end
             if (work(it)==1)then
                j2=it
             else 
                exit A3
             endif 
      enddo A3    
      endif ! if (j1>=1)
      !
      if ((j1>=1) .and. (j2>=1)) Then
         el_diff=elev(j2)-elev(j1)
          if (el_diff<25.d0) Then
             j_start=Min(j2+1,j_end)
                 if (j_start==j_end) Then
                     j2=-1
                     go to 223
                  else
                    j1=-1 ; j2=-1
                    go to 222
                  endif
           endif
      endif

223 continue

      endif !if (istatus==1)
!!!!
!!!!
!
if (istatus==1) then
     if ((j1>=1) .and. (j2>=1)) Then
          ij1=j1 ; ij2=j2
          if (j2==nt-1) ij2=nt
     else
          write(*,*) 'cannot be status==1 and j1,j2 less then 0'
          write(*,*) 'problem in file'
          stop
     endif
! else
!    already defined situation, istatus=-1, ij1=1,ij2=nt
endif

! check now if there are elevations < 0 and larger than 180, if yes remove them
      !---------------------------
      !
       if (elev(ij1)<0) then
              istatus=0
          C1: do ij=ij1,ij2
                 if (elev(ij) >= 0) then
                       ij1=ij
                       exit C1
                 endif
              enddo C1
       endif
      !
       if (elev(ij2)>180) then
               istatus=0
          C2: do ij=ij2,ij1,-1
                 if (elev(ij) <= 180) then
                       ij2=ij
                       exit C2
                 endif
              enddo C2
       endif
      !
      !--------------------------- 


!---------------------------------------------------------------------------------

! if istatus<0 indices from input file are correct, and flipID is eighter 0 or 1
 
if (istatus >=0) Then !  correction for either istatus==0 or istatus==1
! 

            if (flipID==0) Then ! indices j1 and j2 correspond to increasing elevation
               it_s=ij1    +   (raw_data%sweep_start_ray_index(iscan)-1)
               it_e=ij2    +   (raw_data%sweep_start_ray_index(iscan)-1)
               ntimes=it_e-it_s+1
            else if (flipID==1) Then ! flip back so that indices correspond to decreasing elevation 
               it_s=nt-ij2+1         +   (raw_data%sweep_start_ray_index(iscan)-1)
               it_e=nt-ij1+1         +   (raw_data%sweep_start_ray_index(iscan)-1)
               ntimes=it_e-it_s+1
            else
              call write_error('subr CheckElFlippingAndRepeating flipID problem')
              call Exit (1)
            endif

endif
!
111 continue
!write(*,*) 
!write(*,*) 'it_s, it_e,tot', it_s,it_e, it_e-it_s+1
!write(*,*) 'azimuth',nint(minval(raw_data%azimuth(it_s:it_e))),nint((maxval(raw_data%azimuth(it_s:it_e))))
!write(*,*) 'elev', Min(real(raw_data%elevation(it_s)),real(raw_data%elevation(it_e))),Max(real(raw_data%elevation(it_s)),real(raw_data%elevation(it_e))), flipID
!write(*,*) ''
!write(*,*) ''
!
Deallocate(elev)
Deallocate(azimuth)
!
return
end subroutine CheckElFlippingAndRepeating
!!
subroutine masking(raw_data_per_scan,noise,sacr_param)
Use radar_mods
Implicit None
!
Type(sacr_raw_data_per_scan),Intent(InOut)    :: raw_data_per_scan
Type(sacr_noise),Intent(In)                   :: noise
Type(sacr_parameters),Intent(In)              :: sacr_param
!
real*8,parameter                           :: m999=-999.d0
!
integer,Dimension(:,:),Allocatable         :: mask
integer                                    :: itime
!
If (Trim(sacr_param%mode)=='ppimo') Then
call EstimateNoiseFloor_yppi(raw_data_per_scan,noise)
Else
call EstimateNoiseFloor_nppi(raw_data_per_scan,noise,sacr_param)
EndIf
!
! Remove extreme values and adjust Mask1 accordingly
!
Allocate(mask(raw_data_per_scan%ngates,raw_data_per_scan%ntimes))
mask=1
do itime=1,raw_data_per_scan%ntimes
   WHERE (raw_data_per_scan%DBZ(:,itime)<-100) mask(:,itime)=0
   WHERE (abs(raw_data_per_scan%VELO(:,itime))>=60) mask(:,itime)=0
   WHERE (abs(raw_data_per_scan%WIDTH(:,itime))>=20) mask(:,itime)=0   
enddo
!
raw_data_per_scan%Mask1=raw_data_per_scan%Mask1*mask
!
IF (sacr_param%M1ID==0) raw_data_per_scan%Mask1=mask
Deallocate(mask)
!
!! masking using ldr (it redefines mask1)
IF (sacr_param%M2ID==1) call ldr_mask(raw_data_per_scan)
!
! Mask2
Allocate(mask(raw_data_per_scan%ngates,raw_data_per_scan%ntimes))
mask=raw_data_per_scan%Mask1
call masking2(raw_data_per_scan%ngates,raw_data_per_scan%ntimes,mask)
raw_data_per_scan%Mask2=mask
Deallocate(mask)
!
!
IF (sacr_param%M3ID==0) raw_data_per_scan%Mask2=raw_data_per_scan%Mask1 
!

do itime=1,raw_data_per_scan%ntimes
   WHERE (raw_data_per_scan%Mask2(:,itime)==0) 
      raw_data_per_scan%DBZ(:,itime)=m999
      raw_data_per_scan%VELO(:,itime)=m999
      raw_data_per_scan%WIDTH(:,itime)=m999
      raw_data_per_scan%LDR(:,itime)=m999 
      raw_data_per_scan%DBZ_rc(:,itime)=m999
   END WHERE
enddo
!
return
end subroutine masking
!
!
subroutine EstimateNoiseFloor_nppi(raw_data_per_scan,noise,sacr_param)
Use radar_mods
Implicit None
!
Type(sacr_raw_data_per_scan),Intent(InOut)    :: raw_data_per_scan ! observations per scan
Type(sacr_noise),Intent(In)                   :: noise ! structure with mean and std of climatol. noise
Type(sacr_parameters),Intent(In)              :: sacr_param ! input parameter structure 
!
integer                                    :: ntimes ! number of elevations
integer                                    :: ngates ! number of range gates
real*8,Allocatable,Dimension(:)            :: noise_data ! [dB] noise computed by Hildebrand and Sekhon (1974) method
real*8,Allocatable,Dimension(:)            :: noise_clear_int     ![dB]  mean level noise obtained by interpolat.  climat. data 
real*8,Allocatable,Dimension(:)            :: noise_clear_std_int ![dB]  noise std obtained by interpolat.  climat. data
real*8,Allocatable,Dimension(:)            :: noise_data_clean    ![dB]  noise level obtained by combining noise_clear_int and noise_data
real*8                                     :: mean_noise_data_adj ! mean Hildebrand and Sekhon (1974)  noise between select. elevations
real*8                                     :: mean_noise_clear_int_adj ! mean climat. noise between select. elevations
integer                                    :: i1,i2 ! indices of selected min and max elevation to estimate mean noise   
real*8,Allocatable,Dimension(:)            :: refl
real*8                                     :: noise_clear_int_adjusted ! 
real*8                                     :: tr,tr1,tr2
!
integer                                    :: nc
integer                                    :: itime,igate

!
real*8,parameter                           :: m999=-999.d0
!
!
ntimes=raw_data_per_scan%ntimes
ngates=raw_data_per_scan%ngates
!
Allocate(noise_data(ntimes)) ; noise_data=m999  

!Hildebrand and Sekhon (1974)  mean noise calculation for each elevation : noise_data
Allocate(refl(ngates))
N1:do itime=1,ntimes
if (MaxVal(raw_data_per_scan%DBZ_rc(:,itime))==m999) go to 123
refl=0.d0
WHERE(raw_data_per_scan%DBZ_rc(:,itime)/=m999); refl(:)=10.d0**(0.1d0*raw_data_per_scan%DBZ_rc(:,itime)); ENDWHERE
call EstimateNoise(ngates,refl,1,noise_data(itime))
noise_data(itime)=10.d0*dlog10(noise_data(itime))
123 continue
enddo N1 ! itime
Deallocate(refl)
!
! noise for each elevation from climatological noise values
! noise_clear_int & noise_clear_std_int
Allocate(noise_clear_int(ntimes)) ;  noise_clear_int=m999
Allocate(noise_clear_std_int(ntimes)) ;  noise_clear_std_int=m999
!
do itime=1,ntimes
if (MaxVal(raw_data_per_scan%DBZ_rc(:,itime))/=m999) Then
call linpol_sing(noise%elevation,noise%n_std,noise%nn,raw_data_per_scan%elevation(itime),noise_clear_std_int(itime))
call linpol_sing(noise%elevation,noise%n_mean,noise%nn,raw_data_per_scan%elevation(itime),noise_clear_int(itime))
endif
enddo
!
! Adjust the clear sky noise 80-100 degrees elevation to the data noise  80-100 degree elevation
!  since these elevations are the ones we have the most confidence in to find the noise 
!  cause they are the elevations where there is the most clear sky exposed
! This adjustment  is adapted accordingly in other modes

call GetIndicesForSpecifiedValues(ntimes,raw_data_per_scan%elevation,sacr_param,i1,i2)
!
mean_noise_data_adj=0.d0 ; mean_noise_clear_int_adj=0.d0
nc=count(noise_data(i1:i2)/=m999)
if (nc>0) mean_noise_data_adj=Sum(noise_data(i1:i2),MASK=noise_data(i1:i2)/=m999)/dble(nc)
nc=count(noise_clear_int(i1:i2)/=m999)
if (nc>0) mean_noise_clear_int_adj=Sum(noise_clear_int(i1:i2),MASK=noise_clear_int(i1:i2)/=m999)/dble(nc)

! get the noise_data_clean
! Criteria for choosing between data noise of clear sky noise %%%%
Allocate(noise_data_clean(ntimes))
do itime=1,ntimes
if (MaxVal(raw_data_per_scan%DBZ_rc(:,itime))/=m999) Then
noise_clear_int_adjusted = noise_clear_int(itime)  + mean_noise_data_adj -mean_noise_clear_int_adj
tr1=noise_clear_int_adjusted-1.d0*noise_clear_std_int(itime)
tr2=noise_clear_int_adjusted+1.d0*noise_clear_std_int(itime)
!

noise_data_clean(itime) = noise_data(itime)
if ((noise_data(itime)<tr1).or.(noise_data(itime)>tr2)) noise_data_clean(itime) = noise_clear_int_adjusted ! mean climatological value 
!
endif
enddo
!
! First level mask using noise floor (0=noise, 1=signal)
raw_data_per_scan%Mask1=0 ! everything is noise
!
do itime=1,ntimes
if (MaxVal(raw_data_per_scan%DBZ_rc(:,itime))/=m999) Then
   tr=noise_data_clean(itime)+1.d0*noise_clear_std_int(itime)
   do igate=1,ngates
      if (raw_data_per_scan%DBZ_rc(igate,itime)/=m999) Then
         if (raw_data_per_scan%DBZ_rc(igate,itime) > tr) raw_data_per_scan%Mask1(igate,itime)=1 
      endif
   enddo ! igate
endif
enddo ! itime
!
Deallocate(noise_data)
Deallocate(noise_clear_int,noise_clear_std_int)
Deallocate(noise_data_clean)
!
return
end subroutine EstimateNoiseFloor_nppi
!
subroutine  EstimateNoiseFloor_yppi(raw_data_per_scan,noise)
Use radar_mods
Implicit None
!
Type(sacr_raw_data_per_scan),Intent(InOut) :: raw_data_per_scan ! observations per scan
Type(sacr_noise),Intent(In)                :: noise ! structure with mean and std of climatol. noise
!
real*8,Allocatable,Dimension(:)            :: noise_data ! [dB] ! mean noise level obtained using Hildebrand and Sekhon (1974)
real*8,Allocatable,Dimension(:)            :: noise_clear_std_int ! [dB] climatological noise std deviation interpolated at data elevations
real*8                                     :: mean_noise_clear_std_int ! [dB] mean standard deviation of climatological noise  
integer                                    :: ngates ! number of range gates
integer                                    :: ntimes ! number of elevations
!
real*8,Allocatable,Dimension(:)            :: refl
real*8                                     :: tr
integer                                    :: igate,itime
integer                                    :: nc
real*8,parameter                           :: m999=-999.d0
!
!
ngates=raw_data_per_scan%ngates
ntimes=raw_data_per_scan%ntimes
!
Allocate(noise_data(ngates)) 
Allocate(noise_clear_std_int(ntimes))
noise_data=m999 ;  noise_clear_std_int=m999

!Hildebrand and Sekhon (1974)  mean noise calculation for each elevation : noise_data
Allocate(refl(ntimes))
N1:do igate=1,ngates
if (MaxVal(raw_data_per_scan%DBZ_rc(igate,:))==m999) go to 123
refl=0.d0
WHERE(raw_data_per_scan%DBZ_rc(igate,:)/=m999); refl(:)=10.d0**(0.1d0*raw_data_per_scan%DBZ_rc(igate,:)); ENDWHERE
call EstimateNoise(ntimes,refl,1,noise_data(igate))
noise_data(igate)=10.d0*dlog10(noise_data(igate))
123 continue
enddo N1 ! igate
Deallocate(refl)
!
! noise data may be m999
! noise for each elevation from climatological noise values
do itime=1,ntimes
if (MaxVal(raw_data_per_scan%DBZ_rc(:,itime))/=m999) Then
call linpol_sing(noise%elevation,noise%n_std,noise%nn,raw_data_per_scan%elevation(itime),noise_clear_std_int(itime))
endif
enddo
!
mean_noise_clear_std_int=0.d0
nc=count(noise_clear_std_int/=m999)
if (nc>0) mean_noise_clear_std_int=Sum(noise_clear_std_int,MASK=noise_clear_std_int/=m999)/dble(nc)
!
Deallocate(noise_clear_std_int)
!
! First level mask using noise floor (0=noise, 1=signal)
raw_data_per_scan%Mask1=0  
!
do igate= 1,ngates
if (MaxVal(raw_data_per_scan%DBZ_rc(igate,:))/=m999) Then
   do itime=1,ntimes
      if (raw_data_per_scan%DBZ_rc(igate,itime)/=m999) Then
         tr=noise_data(igate)+1.d0*mean_noise_clear_std_int
         if (raw_data_per_scan%DBZ_rc(igate,itime) > tr) raw_data_per_scan%Mask1(igate,itime)=1
      endif
   enddo ! itime
endif
enddo ! igate
!
Deallocate(noise_data)
!
return
end subroutine EstimateNoiseFloor_yppi
!
subroutine masking2(ng,nt,mask)
use radar_mods
Implicit None
!!
Integer,Intent(In)                     :: ng       !! number of radar gates
Integer,Intent(In)                     :: nt       !! number of elevations
Integer,Intent(InOut)                  :: mask(ng,nt)   
!!
Integer,Allocatable,Dimension(:,:)     :: mask1
Integer                                :: iibox
integer                                :: C,Ctr
integer                                :: it,ig
integer                                :: it1,it2,ig1,ig2
integer                                :: iig,iit
integer                                :: l1,l2
integer                                :: iter,niter
real*8,parameter                       :: m999=-999.d0
integer                                :: jgm3,jgp3
!!
!!
!!
Allocate(mask1(ng,nt))
!!
!! first remove the noise pixels using 5x5 box, 5 iterations, if box contains 9 or more cloud pixels it is a cloud pixel
!! 
iibox=2
niter=5
!
mask1=mask
do iter=1,niter
!
      do it=1,nt
            iibox=1
            it1=MAX(1,it-iibox); it2=MIN(it+iibox,nt)
            l1=(it2-it1)+1

            do ig=1,ng
            iibox=3
            ig1=MAX(1,ig-iibox) ; ig2=MIN(ig+iibox,ng)
            l2=(ig2-ig1)+1

               Ctr=(l1*l2 - 1)/3

               IF (mask1(ig,it)==1) Then
                     C=0
                     C=Sum(mask1(ig1:ig2,it1:it2))
                     if (C<=Ctr) mask1(ig,it)=0
               ENDIF
              enddo

         enddo

mask=mask1
enddo
!----------------------------------
!
! now redefine the mask, put back some single in-cloud pixels removed previously as a noise by using H&S74 and/or climat.noise values)
iibox=2
niter=15
!
mask1=mask
do iter=1,niter
!
      do it=1,nt
            it1=MAX(1,it-iibox); it2=MIN(it+iibox,nt)
            l1=(it2-it1)+1

            do ig=1,ng
            ig1=MAX(1,ig-iibox) ; ig2=MIN(ig+iibox,ng)
            l2=(ig2-ig1)+1

               Ctr=l1*l2-1-iibox*2

               IF (mask1(ig,it)==0) Then
                     C=0
                  do iit=it1,it2
                  do iig=ig1,ig2
                     C=C+mask1(iig,iit)
                  enddo
                  enddo 
                     if (C>=Ctr) mask1(ig,it)=1
               ENDIF
              enddo

         enddo

mask=mask1
enddo

Deallocate(mask1)
!
return
end subroutine masking2

subroutine maskingF(ng,nt,mask)
Implicit None
!!
Integer, Intent(In)                    :: ng             !! number of radar gates
Integer, Intent(In)                    :: nt             !! number of elevations
Integer, Intent(InOut)                 :: mask(ng,nt)    !! mask 0=no cloud pixels, 1=cloud pixel
!!
integer                                :: niter  !! number of iterations
integer                                :: ibox   !! ibox=1 for 3x3 box, ibox=2 for 5x5 box
integer                                :: iter,it,ig
integer                                :: iit,iig,it1,it2,ig1,ig2,l1,l2
integer                                :: C,Ctr,C1,C2
Integer,Allocatable,Dimension(:,:)     :: mask1,mask2,mask3
Integer                                :: iibox,iibox1
!
!
!input mask =1 cloud pixels, =0 no cloud pixels
!create mask1 = 1 if all cloud pixels in 5x5 box,
!              =2 all remaining cl. pixels
!              =0 not cloud pixels
iibox=2
!
Allocate(mask1(ng,nt))
 mask1=mask
      do it=1,nt
            it1=MAX(1,it-iibox); it2=MIN(it+iibox,nt)
            l1=(it2-it1)+1

            do ig=1,ng
            ig1=MAX(1,ig-iibox) ; ig2=MIN(ig+iibox,ng)
            l2=(ig2-ig1)+1

               Ctr=l1*l2

               IF (mask(ig,it)==1) Then
                     C=0
                  do iit=it1,it2
                  do iig=ig1,ig2
                     C=C+int(mask(iig,iit))
                  enddo
                  enddo
                     if (C/=Ctr) mask1(ig,it)=2
               ENDIF
               enddo

         enddo

!!-------------------------------
!! input mask1, create mask2
!! we are checking only mask1 =2 pixels
!! mask2=mask1, and mask2=3 for those mask1=2 satisfyning crieria 
!!  1. all surround. pixels in 3x3 box are cloud
!! excluded:  2. mask1=1 for at least one of those surrounding pixels                            
!! so at the end mask2 =1 and mask2=3 are certainly cloud pixels
!! and only remaining mask2=2 are uncertain
iibox=1
Allocate(mask2(ng,nt))
 mask2=mask1
      do it=1,nt
            it1=MAX(1,it-iibox); it2=MIN(it+iibox,nt)
            l1=(it2-it1)+1

            do ig=1,ng
            ig1=MAX(1,ig-iibox) ; ig2=MIN(ig+iibox,ng)
            l2=(ig2-ig1)+1

               Ctr=l1*l2

               IF (mask1(ig,it)==2) Then
                     C=0 ; C1=0
                  do iit=it1,it2
                  do iig=ig1,ig2
                     if (mask1(iig,iit) > 0) C=C+1
                     if (mask1(iig,iit)==1) C1=1
                  enddo
                  enddo
                     !if ((C==Ctr).and.(C1==1)) mask2(ig,it)=3
                     if (C==Ctr)mask2(ig,it)=3
               ENDIF
              enddo

         enddo

!!-------------------------------
!! create mask3,mask3=mask2 ,we check only mask2=2 pixels and look for the noise pixels
!! so mask3 =0 at those mask2=2 pixels if
!! in the box 3x3 there is less than 3 pixels with mask2=3 ('certainly' cloud pixels)  
!! if there is 3 or more  pixels with mask2=3, it is unsertin if it is cloud or noise so
!!  for the middle pixels we check mask values at  3 elevation before and 3 elevation after,
!!  and if total number of cloud pixels <=3 (from in total 7) then it is noise mask3=0
!!    
iibox=1
Allocate(mask3(ng,nt))
mask3=mask2
niter=3

do iter=1,niter
      do it=1,nt
            it1=MAX(1,it-iibox); it2=MIN(it+iibox,nt)
            l1=(it2-it1)+1

            do ig=1,ng
            ig1=MAX(1,ig-iibox) ; ig2=MIN(ig+iibox,ng)
            l2=(ig2-ig1)+1

               Ctr=l1*l2

               IF (mask2(ig,it)==2) Then
                     C=0
                  do iit=it1,it2
                  do iig=ig1,ig2
                     if (mask2(iig,iit) ==3 ) C=C+1                
                  enddo
                  enddo
                     if (C<=2) then
                         mask3(ig,it)=0
                     else
                        iibox1=3; C=0; C1=0 ; C2=0
                        it1=MAX(1,it-iibox1); it2=MIN(it+iibox1,nt)
                        l1=(it2-it1)+1 ; Ctr=(l1-1)/2 
                                          do iit=it1,it2
                                               if (mask2(ig,iit) > 0 ) C=C+1
                                          enddo
                              
                                          do iit=it1,it2
                                               if (mask2(ig1,iit) > 0 ) C1=C1+1
                                          enddo
                                           
                                          do iit=it1,it2
                                               if (mask2(ig2,iit) > 0 ) C2=C2+1
                                          enddo
                       
                                           if ((C<=Ctr).and.(C1<=Ctr)) mask3(ig,it)=0
                                           if ((C<=Ctr).and.(C2<=Ctr)) mask3(ig,it)=0                          

                     endif
               ENDIF
               enddo

         enddo

mask2=mask3
enddo ! iter

!! define output 0-1 mask
mask=mask3
WHERE(mask3>0) ; mask=1 ; ENDWHERE

!! 
iibox=2
niter=5
!
mask1=mask
do iter=1,niter
!
      do it=1,nt
            iibox=1
            it1=MAX(1,it-iibox); it2=MIN(it+iibox,nt)
            l1=(it2-it1)+1

            do ig=1,ng
            iibox=3
            ig1=MAX(1,ig-iibox) ; ig2=MIN(ig+iibox,ng)
            l2=(ig2-ig1)+1

               Ctr=(l1*l2 - 1)/3

               IF (mask(ig,it)==1) Then
                     C=0
                     C=Sum(mask(ig1:ig2,it1:it2))
                     if (C<=Ctr) mask1(ig,it)=0
               ENDIF
              enddo

         enddo

mask=mask1
enddo
!
Deallocate(mask1,mask2,mask3)

! ----------------------------------------------
return
end subroutine maskingF
!
subroutine EstimateNoise(ng,Sn,ns,P)
Use sorting
Implicit None
!
Integer,Intent(in)      :: ng        ! dimension of spectrum vector 
Real*8,Intent(in)       :: Sn(ng)  ! mm^6/m^3  spectrum in linear units
Integer,Intent(in)      :: ns ! number of points over which a moving average of the spectrum was taken
                              ! ns=1 if the spectrum has not been smoothed
Real*8,Intent(Out)      :: P  ! mm^6/m^3 the mean noise level P for the spectrum Sn below R2
!
Real*8                  :: Q  !  [mm^6/m^3] the variance of the spectral values below R2
Integer                 :: n  !   the number of points in the spectrum below R2
Real*8                  :: SnR2  ! the signal-to-noise critical threshold 
                                 ! SnR2 is the critical value where the criterion for white noise has been met, 
                                 ! while P is the mean of all the values in Sn below this critical value.
Real*8                  :: maxSn ! [mm^6/m^3] the maximum value in the spectrum maxSn
!!
Real*8                               :: R2
Real*8,Allocatable,Dimension(:)      :: work
integer                              :: i1(1),i2
integer                              :: i
!! For more information, please see  Hildebrand and Sekhon (1974)
!!

maxSn = MaxVal(Sn)

Allocate(work(ng))
work=Sn
call Sort(work,ng) ! sorts into an ascending order
work(1:ng)=work(ng:1:-1) ! sorted into descending order
i1=MinLoc(work,MASK=work>0.d0)
i2=i1(1)
SnR2 = work(i2) ! Set signal-to-noise threshold to lowest value in spectrum.
!
A1: do i = 1,ng
       n = count(work(i:i2)>0.d0)
       P = Sum(work(i:i2))/dble(n)  ! Mean noise level for current iteration.
       Q = (Sum(work(i:i2)**2))/dble(n) - P**2  ! Variance of noise level for current iteration.
       R2 = P**2 / (Q * dble(ns))
         if (R2 > 1) Then  ! White noise criteria has been met.
             SnR2 = work(i)
             Exit A1
         endif
      end do A1
!
Deallocate(work)
!
!
return
end subroutine EstimateNoise
!!
!!
subroutine GetIndicesForSpecifiedValues(nt,elev,sacr_param,i1,i2)
Use radar_mods
Use write_messages
Implicit None
!
integer,Intent(In)                 :: nt
real*8, Intent(In)                 :: elev(nt)
type(sacr_parameters),Intent(In)   :: sacr_param
Integer,Intent(Out)                :: i1,i2
!!
real*8                             :: max_val,min_val
integer                            :: it
!!
i1=0 ; i2=0
!
! for cwrhi and hsrhi
max_val=100.d0
min_val=80.d0 
if (Trim(sacr_param%mode)=='blrhi') Then
max_val=MaxVal(elev)
min_val=MIN(max_val-5.d0,75.d0)
endif
!
!
If (Trim(sacr_param%mode)=='vptmo')Then
i2=idnint(dble(nt)/2.d0)+idnint(dble(nt)/4.d0)
i1=idnint(dble(nt)/2.d0)-idnint(dble(nt)/4.d0)
Else
!
A1:do it=1,nt
  if ((elev(it)-min_val)>=0.d0) Then
   i1=it
   Exit A1
  endif
enddo A1
if (abs(elev(i1)-min_val)>abs(elev(i1-1)-min_val)) i1=i1-1
!
A2:do it=1,nt
  if ((elev(it)-max_val)>=0.d0) Then
   i2=it
   Exit A2
  endif
enddo A2
!
if (abs(elev(i2)-max_val)>abs(elev(i2-1)-max_val)) i2=i2-1
!
endif
!!
!!
if ((i1<0).or.(i2<0))Then
call write_error('problem i1,i2 <0, subr. GetIndicesForSpecifiedValues')
call Exit (1)
endif

return
end subroutine GetIndicesForSpecifiedValues
!
!!
!!
subroutine curvature_correction(raw_data_per_scan)
use radar_mods
Implicit None
!
Type(sacr_raw_data_per_scan),Intent(InOut)           :: raw_data_per_scan
!
real*8,parameter                           :: Re=6371.d0*1.d+3 !m
real*8                                     :: pi
real*8                                     :: p_r,p_r2
integer                                    :: ng, nt
integer                                    :: k, j
real*8                                     :: a,b,c,d,e
real*8                                     :: f1,f2,f3,f4

pi=dasin(1.0d+0)*2.d0
p_r=4.d0*Re/3.d0   ! m

ng=raw_data_per_scan%ngates
nt=raw_data_per_scan%ntimes

p_r2=p_r**2

do k=1,ng
do j=1,nt

a=raw_data_per_scan%range(k)**2 + p_r2
b=2.d0*raw_data_per_scan%range(k)*p_r
c=b* dsin(raw_data_per_scan%elevation(j)*pi/180.d0)
d=(a + c)**0.5d0
e=raw_data_per_scan%range(k)*dcos(raw_data_per_scan%elevation(j)*pi/180.d0)
!
f1=d -p_r
f2=p_r*dasin( e / d )
!
f3=(f1**2 + f2**2)**0.5d0
!

f4=datan(f1/f2)*180/pi
if (f4<0) f4=180.d0+f4
!
!
raw_data_per_scan%z_distance(k,j)=f1  !m
raw_data_per_scan%x_distance(k,j)=f2  ! assumed azimuth==0
!
raw_data_per_scan%range_corrected(k,j)=f3
raw_data_per_scan%elevation_corrected(k,j)=f4
!
enddo
enddo
!
return
end subroutine curvature_correction
!!
subroutine GetSoundingID(nsoundings,sounding_data_file,time_string,is_true,dt_sounding_scan)
Implicit None
!
integer,Intent(In)                          :: nsoundings
Character(len=*),Intent(In)                 :: sounding_data_file(nsoundings)
Character(len=*),Intent(In)                 :: time_string
integer,Intent(Out)                         :: is_true ! index of the closest sounding to scan
real*8,Intent(Out)                          :: dt_sounding_scan ! time interval between sounding time and sacr scan time in seconds
!
Character(len=200)                          :: sfilename
integer                                     :: is
integer                                     :: index_pos
character(len=30),Dimension(:),Allocatable  :: stime
character(len=2)                            :: hh,mm,ss
integer                                     :: ihh,imm,iss
integer,Dimension(:),Allocatable            :: itime
character(len=30)                           :: true_stime
integer                                     :: true_itime
integer                                     :: diff,work

Allocate(stime(nsoundings),itime(nsoundings))

! Identify the closest sounding to the scan
do is =1,nsoundings
call get_filename(sounding_data_file(is),sfilename)
index_pos=INDEX(trim(sfilename),'.cdf')
stime(is)=sfilename((index_pos-6):index_pos-1)
hh=sfilename((index_pos-6):index_pos-5)
mm=sfilename((index_pos-4):index_pos-3)
ss=sfilename((index_pos-2):index_pos-1)
read (hh,*) ihh
read (mm,*) imm
read (ss,*) iss
itime(is)=ihh*3600 + imm*60 + iss
enddo

true_stime=time_string(10:15)
hh=time_string(10:11)
mm=time_string(12:13)
ss=time_string(14:15)
read (hh,*) ihh
read (mm,*) imm
read (ss,*) iss
true_itime=ihh*3600  + imm*60  + iss

diff=abs(true_itime-itime(1))
is_true=1
do is=1,nsoundings
   work=abs(true_itime-itime(is))
     if (work<diff) Then
         is_true=is
         diff=work
     endif
enddo
!
dt_sounding_scan=dble(abs(true_itime-itime(is_true)))
Deallocate(stime,itime)
!
return
end subroutine GetSoundingID
!
subroutine InsectRemoval(ceilo_data,sounding,rad_syst,sacr_param,raw_data_per_scan)
use radar_mods
Implicit None
!
Type(sacr_ceilo_data),Intent(In)                     :: ceilo_data
Type(sacr_sounding_data),Intent(In)                  :: sounding
Type(sacr_system_constants),Intent(In)               :: rad_syst
Type(sacr_parameters),Intent(In)                     :: sacr_param
Type(sacr_raw_data_per_scan),Intent(InOut)           :: raw_data_per_scan
!
real*8                                               :: ceilo_clb ! [m] the first ceilo data cloud base above MSL 
                                                                  !     corresponding to the sacr scan time

real*8                                               :: isoth5_height ! [m] the height above MSL of 5C isotherm from sounding
real*8,parameter                                     :: m999=-999.d0
real*8                                               :: zthr! =min(ceilo_clb,isoth5_height) in sacr radar coord. system (0=altitude of radar)
real*8,Dimension(:),Allocatable                      :: LDR
integer                                              :: ng, nt
integer                                              :: ig, it
!
!
!-------------------------------------------------------------------------------
zthr=m999
isoth5_height=m999
ceilo_clb=m999
!
if (sounding%soundeID==1) then ! sounding data present
   call Find5IsothermHeight(sounding,isoth5_height)
   if (isoth5_height>0) zthr=isoth5_height
endif
!
if (ceilo_data%ceiloID==1) then ! ceilo data present
call FindCeiloFirstCloudBase(ceilo_data,raw_data_per_scan%time_str,ceilo_clb)
   if (ceilo_clb>0) zthr=ceilo_clb
endif
!
if ((ceilo_clb>0).and.(isoth5_height>0)) zthr=MIN(ceilo_clb,isoth5_height)
!-------------------------------------------------------------------------------
!!
if (zthr/=m999) then
   zthr=zthr-(rad_syst%altitude_sea_level(1)+rad_syst%altitude_from_ground(1))
   !
   ng=raw_data_per_scan%ngates
   nt=raw_data_per_scan%ntimes
   Allocate(LDR(ng))
   do it=1,nt ! for each elevation
     LDR(:)=raw_data_per_scan%LDR(:,it)
       do ig=1,ng
          if (raw_data_per_scan%z_distance(ig,it)<=zthr) Then
            if (LDR(ig)>sacr_param%ir_ldr_thresh) Then ! conservative threshold for insects LDR  -5, valid range -5 to -15
                raw_data_per_scan%DBZ(ig,it)   =m999
                raw_data_per_scan%VELO(ig,it)  =m999
                raw_data_per_scan%LDR(ig,it)   =m999
                raw_data_per_scan%WIDTH(ig,it) =m999
            endif  
          endif
       enddo  ! ig
   enddo  ! it
   Deallocate(LDR)
endif
!
!
return
end subroutine InsectRemoval
!
subroutine FindCeiloFirstCloudBase(ceilo_data,time_string,ceilo_clb)
use radar_mods
Implicit None
!
Type(sacr_ceilo_data),Intent(In)                     :: ceilo_data
Character(len=*),Intent(In)                          :: time_string
Real*8, Intent(Out)                                  :: ceilo_clb ! [m] the first ceilo data cloud base from ground
!                                                                  !     corresponding to the sacr scan time
!
character(len=30)                           :: true_stime
integer                                     :: true_itime
character(len=2)                            :: hh,mm,ss
integer                                     :: ihh,imm,iss
!
character(len=30)                           :: ctime_str
integer*4                                   :: t(9)
integer,Dimension(:),Allocatable            :: ceilo_time ! ceilo time in seconds from midnight 
integer                                     :: it
integer                                     :: diff,work
integer                                     :: it_true
integer                                     :: it1,it2
integer                                     :: mtime,ptime
integer                                     :: nn
real*8,parameter                            :: m999=-999.d0
!
!! sacr time scan in seconds  
hh=time_string(10:11)
mm=time_string(12:13)
ss=time_string(14:15)
read (hh,*) ihh
read (mm,*) imm
read (ss,*) iss
true_itime=ihh*3600 + imm*60 + iss
!
Allocate(ceilo_time(ceilo_data%nt))
!! ceilo time in seconds
do it=1,ceilo_data%nt
call EpochTime2ReadableFormat(int(ceilo_data%time_s(it),4),t)
ctime_str=''
write(ctime_str,'(a,(I4.4,I2.2,I2.2,"_",I2.2,I2.2,I2.2))') trim(ctime_str),t(6)+1900, t(5)+1, t(4), t(3), t(2), t(1)
ctime_str=Trim(ctime_str) ! in format YYYYMMDD_HHMMSS
!
hh=ctime_str(10:11)
mm=ctime_str(12:13)
ss=ctime_str(14:15)
read (hh,*) ihh
read (mm,*) imm
read (ss,*) iss
ceilo_time(it)=ihh*3600 + imm*60 + iss
enddo
!
!
! find the index of the closest ceilo time to the sacr scanning time
diff=abs(true_itime-ceilo_time(1))
it_true=1
do it=1,ceilo_data%nt
   work=abs(true_itime-ceilo_time(it))
   if (work<diff) then
      it_true=it
      diff=work
   endif
enddo
!
!----
! find the indices half of hour before and half of hour later than ceilo_time(it_true)
it1=1 ; it2=ceilo_data%nt
!   it1 
if (ceilo_time(it_true)>180) Then   !180 =30 min* 60 sec, if ceilo_time(it_true) later than 00:30:00
    mtime=ceilo_time(it_true)-180   ! -0.5 h
    diff=abs(mtime-ceilo_time(1))
    it1=1
    do it=1,ceilo_data%nt
       work=abs(mtime-ceilo_time(it))
         if (work<diff) Then
            it1=it
            diff=work
         endif
    enddo
endif
!   it2
if (ceilo_time(it_true)<84600) Then ! 84600=23*3600 + 30*60
   ptime=ceilo_time(it_true)+180    ! +0.5 h
   diff=abs(ptime-ceilo_time(1)) 
    it2=1
    do it=1,ceilo_data%nt
       work=abs(ptime-ceilo_time(it))
         if (work<diff) Then
             it2=it
             diff=work
          endif
    enddo
endif
!
Deallocate(ceilo_time)
ceilo_clb=m999

nn=count(ceilo_data%first_cbh(it1:it2)>=0.d0)
if (nn>0) Then
ceilo_clb=Sum(ceilo_data%first_cbh(it1:it2),MASK=ceilo_data%first_cbh(it1:it2)>=0.d0)/dble(nn)
ceilo_clb=ceilo_clb+ceilo_data%alt ! now above MSL
endif
!
return
end subroutine FindCeiloFirstCloudBase
!
subroutine Find5IsothermHeight(sounding,isoth5_height)
use radar_mods
Implicit None
Type(sacr_sounding_data),Intent(In)                  :: sounding
Real*8,Intent(Out)                                   :: isoth5_height ! [m] the height of 5 C isotherm from sounding above MSL
!
integer                                              :: is,ns
integer                                              :: is5
real*8                                               :: work,diff
real*8,parameter                                     :: m999=-999.d0


isoth5_height=m999
is5=-1
!
ns=sounding%sounde
diff=abs(sounding%Temperature(1)-5.d0)
do is=1,ns
   work=abs(sounding%Temperature(is)-5.d0)
if (work<diff) Then
   is5=is
   diff=work
endif
enddo   
!
if (is5>0) isoth5_height=sounding%altitude(is5)
!
return
end subroutine Find5IsothermHeight
!
subroutine RemoveSecondTripEcho(sacr_param,raw_data_per_scan)
use radar_mods
Implicit None
!
Type(sacr_parameters),Intent(In)                     :: sacr_param
Type(sacr_raw_data_per_scan),Intent(InOut)           :: raw_data_per_scan
!
integer                                              :: ng,nt
integer                                              :: nel
integer,Dimension(:,:), Allocatable                  :: mask
Real*8,Dimension(:,:) , Allocatable                  :: vel
integer                                              :: it, ig
integer                                              :: niter,ibox
!
real*8                                               :: el_th1,el_th2
real*8,parameter                                     :: m999=-999.d0

ng=raw_data_per_scan%ngates
nt=raw_data_per_scan%ntimes
!
el_th1=sacr_param%ste_el_min
el_th2=sacr_param%ste_el_max
!--------------------------------------------------------------------------
nel=count((raw_data_per_scan%elevation<el_th1).or.(raw_data_per_scan%elevation>el_th2))
!
if (nel>0) Then
      Allocate(mask(ng,nt)) ; mask=0
      Allocate(vel(ng,nt)); vel=m999
      !
      vel(1:ng,1:nt)=raw_data_per_scan%VELO(1:ng,1:nt)
      !
      do it=1,nt
         if ((raw_data_per_scan%elevation(it)<el_th1).or.(raw_data_per_scan%elevation(it)>el_th2)) Then
         WHERE (raw_data_per_scan%VELO(:,it)/=m999)  mask(:,it)=1
         endif
      enddo
      !
      ! vel=VELO ; mask=1 only in cloud pixels at el<el_th1 and >el_th2
      niter=1 ; ibox=1
      call masking_ste(niter,ibox,ng,nt,vel,sacr_param%ste_vel_thresh,mask)    
      !
      Deallocate(vel)
      !

         do it=1,nt
            if ((raw_data_per_scan%elevation(it)<el_th1).or.(raw_data_per_scan%elevation(it)>el_th2)) Then
               WHERE (mask(:,it)==0) 
                  raw_data_per_scan%DBZ(:,it)=m999
                  raw_data_per_scan%VELO(:,it)=m999
                  raw_data_per_scan%WIDTH(:,it)=m999
                  raw_data_per_scan%LDR(:,it)=m999  
               END WHERE
            endif
         enddo
      !
      Deallocate(mask)
endif
!--------------------------------------------------------------------------
!
return
end subroutine RemoveSecondTripEcho
!
subroutine masking_ste(niter,ibox,ng,nt,vel,ste_vel_thresh,mask)
Implicit None
!!
Integer, Intent(In)                    :: niter  !! number of iterations
Integer, Intent(In)                    :: ibox   !! ibox=1 for 3x3 box, ibox=2 for 5x5 box
Integer, Intent(In)                    :: ng     !! number of radar gates 
Integer, Intent(In)                    :: nt     !! number of elevations
Real*8,  Intent(In)                    :: vel(ng,nt)        !! velocity
Real*8,  Intent(In)                    :: ste_vel_thresh    !! treshold value of velocity standard deviation in m/s for the second trip echo identification
Integer, Intent(InOut)                 :: mask(ng,nt)       !! mask  for vel for el < 20 and el > 160 (0=no cloud pixels, 1=cloud pixel)
                                                            !! for all el >20 < 160 mask=0
!!
integer                                :: iter,it,ig
integer                                :: iit,iig,it1,it2,ig1,ig2,l1,l2
integer                                :: C,Ctr
Integer,Allocatable,Dimension(:,:)     :: mask1
real*8                                 :: vstd,vmean
real*8                                 :: ste_vel_thresh2
real*8,Parameter                       :: m999=-999.d0
!
!
ste_vel_thresh2=ste_vel_thresh*ste_vel_thresh ! velocity variance threshold

Allocate(mask1(ng,nt))
!
!
do iter=1,niter
 mask1=mask
 mask=0

            do ig=1,ng
            ig1=MAX(1,ig-ibox) ; ig2=MIN(ig+ibox,ng)
            l2=(ig2-ig1)+1

      do it=1,nt
            it1=MAX(1,it-ibox); it2=MIN(it+ibox,nt)
            l1=(it2-it1)+1


               IF (mask1(ig,it)==1) Then
                     C=0 ; vmean=0.d0
                  C=count(vel(ig1:ig2,it1:it2)/=m999)
                  if (C>=2) then
                      vmean=Sum(vel(ig1:ig2,it1:it2),MASK=vel(ig1:ig2,it1:it2)/=m999)                
                      vmean=vmean/dble(C)
                           vstd=0.d0
                           do iig=ig1,ig2 
                           do iit=it1,it2
                              if (vel(iig,iit)/=m999) vstd=vstd+ (vel(iig,iit)-vmean)*(vel(iig,iit)-vmean)
                            enddo
                            enddo
                       vstd=vstd/dble(C-1) 
                       if (vstd<=ste_vel_thresh2) mask(ig,it)=mask1(ig,it)  !   ! it IS NOT second trip echo
                    endif ! if C>=2 
                ENDIF ! IF (mask1(ig,it)==1)

              enddo ! ig

          enddo ! it
!!
enddo ! iter
!!
Deallocate(mask1)
return
end subroutine masking_ste
!!
!!
subroutine EstimateGaseousAttenuation(fr,sounding,rad_syst,raw_data_per_scan)
use radar_mods
use write_messages
Implicit None
!
real*8,Intent(In)                                    :: fr ! frequency in GHz
Type(sacr_sounding_data),Intent(In)                  :: sounding
Type(sacr_system_constants),Intent(In)               :: rad_syst
Type(sacr_raw_data_per_scan),Intent(InOut)           :: raw_data_per_scan
!
integer                  :: nt,ng,ns
integer                  :: it,ig
!
real*8,Dimension(:),Allocatable    :: zz   ! [m]  above MSL
real*8,Dimension(:),Allocatable    :: temp ! [C]
real*8,Dimension(:),Allocatable    :: rh   ! [%]
real*8,Dimension(:),Allocatable    ::press ! [hPa]
!
!
real*8                   :: tot_tau
real*8                   :: dr  ! delta range in m
real*8                   :: att ! extinction by dry air and water vapor in Np/m estimated by  ROSENKRANZ (1998) model
real*8                   :: loc_tau
real*8                   :: dz_rad  ! radar altitude above MSL in m
real*8                   :: work
real*8,Parameter         :: m999=-999.d0
integer                  :: istatus

nt=raw_data_per_scan%ntimes
ng=raw_data_per_scan%ngates
ns=sounding%sounde
!
dr=raw_data_per_scan%range(3)-raw_data_per_scan%range(2)  ! m
dz_rad=rad_syst%altitude_sea_level(1)+ rad_syst%altitude_from_ground(1)  ! m
!
!
istatus=1
If (MinVAl(sounding%altitude)==m999) istatus=0
If (MinVAl(sounding%temperature)==m999) istatus=0
If (MinVAl(sounding%relative_hum)==m999) istatus=0
if (MinVAl(sounding%pressure)==m999) istatus=0
!
if (istatus==0) Then
call write_error('subr: EstimateGaseousAttenuation: Problem with missing values in soundings')
call Exit (1)
endif
!
do it=1,nt
   ! 
   ! interpolate temperature, rel. humidity and pressure
   Allocate(zz(ng),temp(ng),rh(ng),press(ng))
   zz(:)=raw_data_per_scan%z_distance(:,it)+dz_rad  ! above MSL
   call linpol(sounding%altitude,sounding%temperature,ns,zz,temp,ng)
   call linpol(sounding%altitude,sounding%relative_hum,ns,zz,rh,ng)
   call linpol(sounding%altitude,sounding%pressure,ns,zz,press,ng)
   !
   tot_tau=0.d0
   call rosen98_gasabs(fr,temp(1),rh(1),press(1),att) 
   loc_tau=att*raw_data_per_scan%range(1) ! Np/m *m =Np
   tot_tau=tot_tau +loc_tau
     if (raw_data_per_scan%dBZ(1,it)/=m999) then
        work=10.d0**(0.1d0*raw_data_per_scan%dBZ(1,it)) ! mm^6/m^3
        raw_data_per_scan%dBZ_cor(1,it)=10.d0*dlog10(work*dExp(2.d0*tot_tau))
      endif
   do ig=2,ng
         !
         call rosen98_gasabs(fr,temp(ig),rh(ig),press(ig),att)
         loc_tau=att*dr !  Np/m *m =Np
         tot_tau=tot_tau+loc_tau 
           if (raw_data_per_scan%dBZ(ig,it)/=m999) then
               work=10.d0**(0.1d0*raw_data_per_scan%dBZ(ig,it)) ! mm^6/m^3 
               raw_data_per_scan%dBZ_cor(ig,it)=10.d0*dlog10(work*dExp(2.d0*tot_tau))
           endif
         !
   enddo
   !
   Deallocate(zz,temp,rh,press)   
   !
enddo
!
!
return
end subroutine EstimateGaseousAttenuation

subroutine rosen98_gasabs(fr,temp,rh,press,rosen98_att)
use gasabs_module
Implicit None
!
real*8,Intent(In)         :: fr           ! [GHz]   frequency
real*8,Intent(In)         :: temp         ! [C]     temperature
real*8,Intent(In)         :: press        ! [mb]    pressure
real*8,Intent(In)         :: rh           ! [%]     relative humidity
real*8,Intent(Out)        :: rosen98_att  ! [Np/m] extinction by dry air and water vapor
!!
real*8        :: absair ! [Np/km] extinction by dry air
real*8        :: abswv  ! [Np/km] extinction by water vapor
!!
real*8        :: tempK ! temperature in K
real*8        :: es    ! saturation vapor pressure in Pa
real*8        :: e     ! vapor pressure in Pa
real*8        :: wv    ! vapor density in g/m3
real*8,parameter       :: Rv=461.5d0 ! J /(kg K)
integer       :: istatus
real*8,parameter       :: m999=-999.d0
!!
!!
istatus=1

if (temp==m999) istatus=0
if (press==m999) istatus=0
if (rh==m999) istatus=0


if (istatus==1) then

tempK=temp+273.15d0  ! temperature in K
es=( 6.112d0*dexp((17.67d0*temp)/(temp+243.5d0))) *1.d+2 ! saturation vapor pressure in Pa
e=es*rh*1.d-2   ! vapor pressure in Pa
wv=e/(Rv*tempK) ! water vapor density in kg/m^3 (from ideal gas low)
!
wv =wv*1.d+3 ! kg/m^3 -> gr/m^3
!
! get volume extinction coefficients
!
absair = absn2(tempK,press,fr)    + o2abs(tempK,press,wv,fr)
abswv  = abh2o(tempK,press,wv,fr)
!
rosen98_att= (absair + abswv)*1.d-3   ! Np/km -> Np/m
!
Else 
!
rosen98_att=0.d0
!
Endif

return
end subroutine rosen98_gasabs
!!
subroutine dealiasing(rad_syst,sounding,raw_data_per_scan,folding)
Use radar_mods
Implicit None
Type(sacr_system_constants),Intent(In)    :: rad_syst
Type(sacr_sounding_data),Intent(In)       :: sounding
Type(sacr_raw_data_per_scan),Intent(In)   :: raw_data_per_scan
Type(sacr_folding_vars),Intent(InOut)     :: folding
!
integer                                   :: ns,ng,nt
real*8                                    :: dz_rad ! radar instrument altitude above MSL
!
real*8,Dimension(:),Allocatable           :: wind_dir,wind_speed,d_azimuth
!       
real*8,Dimension(:),Allocatable           :: dbz_doppler_threshold
real*8,Dimension(:),Allocatable           :: VD_unfolded_BUpass
real*8,Dimension(:),Allocatable           :: VD_unfolded_TDpass
!
integer                                   :: fold_flagBU
integer                                   :: fold_flagTD
real*8                                    :: cfIN
real*8                                    :: cfBU
real*8                                    :: cfTD
real*8,Dimension(:),Allocatable           :: cl_pix_VD
real*8,Dimension(:),Allocatable           :: work
integer,Dimension(:),Allocatable          :: cl_pix_VD_index
real*8                                    :: diff_VD,diff
real*8                                    :: work1

integer                                   :: k
integer                                   :: k1
integer                                   :: nk
integer                                   :: ktr

real*8                                    :: pif  ! pi value devided by 180 degrees
integer                                   :: it,ig
real*8,parameter                          :: m999=-999.d0

pif=(dasin(1.0d+0)*2.d0)/180.d0
!
ns=sounding%sounde
ng=raw_data_per_scan%ngates
nt=raw_data_per_scan%ntimes
!
dz_rad=rad_syst%altitude_sea_level(1)+ rad_syst%altitude_from_ground(1)  ! m
!
Allocate(wind_dir(ng))
Allocate(wind_speed(ng))
Allocate(d_azimuth(ng))

do it=1,nt
!
      call linpol(sounding%altitude,sounding%wd,ns,raw_data_per_scan%z_distance(:,it)+dz_rad,wind_dir,ng)
      call linpol(sounding%altitude,sounding%wf,ns,raw_data_per_scan%z_distance(:,it)+dz_rad,wind_speed,ng)
!
      folding%wf(:,it) = wind_speed(:)
      folding%wd(:,it) = wind_dir(:)
!
      d_azimuth(:)  = raw_data_per_scan%azimuth(it)  - wind_dir(:)
!
      !  Project wind speed from sounding data on different planes
      folding%Up_az(:,it)    = wind_speed(:) * dcos(d_azimuth(:)*pif)
      folding%Up_90(:,it)    = wind_speed(:) * dcos((d_azimuth(:)+90.d0)*pif)
      folding%Up_az_el(:,it) = folding%Up_az(:,it) * dcos(raw_data_per_scan%elevation_corrected(:,it)*pif)
      ! 
      WHERE (raw_data_per_scan%VELO(:,it) /= m999) 
      ! Calculate number of aliasing based on wind projection on the scanning plane
      folding%aliasn(:,it)   = idnint((folding%Up_az_el(:,it)-raw_data_per_scan%VELO(:,it))/(2.d0*rad_syst%nyquist_velo(1)))
      ! Initial de_aliasing
      folding%VD_unfolded(:,it) = raw_data_per_scan%VELO(:,it) + dble(folding%aliasn(:,it))* 2.d0*rad_syst%nyquist_velo(1)
      END WHERE
      !  
enddo
!
Deallocate(wind_dir)
Deallocate(wind_speed)
Deallocate(d_azimuth)
!!
!!
!! Doppler velocity quality mask based on reflectivity
Allocate(dbz_doppler_threshold(ng))
Allocate(VD_unfolded_BUpass(ng))
Allocate(VD_unfolded_TDpass(ng))
!

do it=1,nt
      !
      ! initialisation
      dbz_doppler_threshold=m999
      VD_unfolded_BUpass=folding%VD_unfolded(:,it)
      VD_unfolded_TDpass=folding%VD_unfolded(:,it)
      fold_flagBU=0 
      fold_flagTD=0
      !
      if (MaxVal(raw_data_per_scan%DBZ(:,it))==m999) go to 123
      !
      ! create range dependent reflectivity threshold value
      do ig=1,ng
         if (raw_data_per_scan%range_corrected(ig,it)/=m999) Then 
            dbz_doppler_threshold(ig)= -40.d0+20.d0*dlog10(raw_data_per_scan%range_corrected(ig,it)*1.d-3)
         endif
      enddo
      !remove velocity cloud pixels with low reflectivity (dbz<dbz_threshold)
      do ig=1,ng      
         if ((raw_data_per_scan%DBZ(ig,it)/=m999).and.(raw_data_per_scan%DBZ(ig,it)<dbz_doppler_threshold(ig))) &
            VD_unfolded_BUpass(ig)=m999       
      enddo
      ! count the number of remained cloud pixels  
      nk=0
      nk=count(VD_unfolded_BUpass/=m999)
      !
      if (nk<=1) folding%VD_unf_flag(it)=1 ! no pixels with high reflectivity so initial folding correction cannot be checked out
      if (nk<=1) go to 123
      !
      ! create cloud-only velocity arrays where reflectivity dbz>=dbz_threshold
      Allocate(cl_pix_VD(nk)) ; cl_pix_VD=m999
      Allocate(cl_pix_VD_index(nk))  ; cl_pix_VD_index=0
      k=0
      do ig=1,ng
         if (VD_unfolded_BUpass(ig)/=m999) then
            k=k+1
            cl_pix_VD(k) = VD_unfolded_BUpass(ig)
            cl_pix_VD_index(k) = ig
         endif
      enddo
      !
      ! find if there are two adjacent cloud pixels that have velocities difference above the nyquist 
      ! (indicator of bad unfolding)
      diff=abs(cl_pix_VD(2)-cl_pix_VD(1))
      do k=1,nk-1
        diff_VD=abs(cl_pix_VD(k)-cl_pix_VD(k+1))
        if (diff_VD>diff) diff=diff_VD
      enddo
      !
      if (diff<=rad_syst%nyquist_velo(1)) folding%VD_unf_flag(it)=1 ! initial folding correction works, no differencies above the nyquist
      if (diff<=rad_syst%nyquist_velo(1)) go to 121
      !
      ! reinitialize VD_unfolded_TDpass
      VD_unfolded_TDpass=VD_unfolded_BUpass
      ! 
      Allocate(work(nk))
      !
      ! loop through the cloud pixels
      ! find if there are two adjacent cloud pixels that have velocities difference above the nyquist
      ! and if so ajust the pixel ABOVE to match the pixel BELLOW with a difference smaller than the nyquist velocity
            !
            work=cl_pix_VD
            do k=1,nk-1
                  diff_VD=work(k)-work(k+1)
                  if (diff_VD >  rad_syst%nyquist_velo(1)) Then
                      work(k+1)=work(k+1)+ 2.d0*rad_syst%nyquist_velo(1)
                      fold_flagBU=1
                  else if (diff_VD < -rad_syst%nyquist_velo(1)) Then
                      work(k+1)=work(k+1)- 2.d0*rad_syst%nyquist_velo(1)
                      fold_flagBU=1
                  endif
            enddo
            !
            ! adjust the VD_unfolded_BUpass accordingly
            if (fold_flagBU==1) Then
              do k=1,nk
                    ig=cl_pix_VD_index(k)
                    VD_unfolded_BUpass(ig)=work(k)
              enddo
            endif
           !
      ! loop through the cloud pixels
      ! find if there are two adjacent cloud pixels that have velocities difference above the nyquist
      ! and if so ajust the pixel BELLOW to match the pixel ABOVE with a difference smaller than the nyquist velocity
        
            work=cl_pix_VD
            do k=nk,2,-1
                  diff_VD=work(k)-work(k-1)
                  if (diff_VD >  rad_syst%nyquist_velo(1)) Then
                      work(k-1)=work(k-1)+ 2.d0*rad_syst%nyquist_velo(1)
                      fold_flagTD=1
                  else if (diff_VD < -rad_syst%nyquist_velo(1)) Then
                      work(k-1)=work(k-1)- 2.d0*rad_syst%nyquist_velo(1)
                      fold_flagTD=1
                  endif
            enddo
            !
            ! adjust the VD_unfolded_TDpass accordingly
            if (fold_flagTD==1) Then
              do k=1,nk
                    ig=cl_pix_VD_index(k)
                    VD_unfolded_TDpass(ig)=work(k)
              enddo
            endif
       !
       Deallocate(work)   
       !
       ! Add the initial unfolding values in cloud pixels with low reflectivities (dbz<dbz_threshold)
       do ig=1,ng
           if ((raw_data_per_scan%DBZ(ig,it)/=m999).and.(raw_data_per_scan%DBZ(ig,it)<dbz_doppler_threshold(ig))) Then
             VD_unfolded_BUpass(ig)=folding%VD_unfolded(ig,it)
             VD_unfolded_TDpass(ig)=folding%VD_unfolded(ig,it)
           endif
       enddo
       !

121 continue 
       !
       Deallocate(cl_pix_VD,cl_pix_VD_index)
       !

if ((fold_flagBU==0).and.(fold_flagTD==0)) go to 123    ! it goes to 123 when it is found that initial folding correction works, no differencies above the nyquist
   !
   ! Choose between the original unfolding and both BU and TD unfolded velocity estimates using a confidence test
   call compute_cost_function_for_folding(ng,folding%Up_az_el(:,it),folding%VD_unfolded(:,it),cfIN)
   !
   cfBU=cfIN
   cfTD=cfIN
   folding%VD_unf_flag(it)=1 ! initial corrction works better than the BU and TD filters; if not, will be overwritten
   !
   if (fold_flagBU==1) call compute_cost_function_for_folding(ng,folding%Up_az_el(:,it),VD_unfolded_BUpass,cfBU) 
   if (fold_flagTD==1) call compute_cost_function_for_folding(ng,folding%Up_az_el(:,it),VD_unfolded_TDpass,cfTD)
   !
   !
   if (cfBU > cfTD) Then 
      if (cfIN > cfTD) Then
         folding%VD_unfolded(:,it)=VD_unfolded_TDpass(:)
         folding%VD_unf_flag(it)=2  ! correction of the initial unfolding using top-to-bottom pass
      endif
   else
      if (cfIN > cfBU) Then
        folding%VD_unfolded(:,it)=VD_unfolded_BUpass(:)
        folding%VD_unf_flag(it)=3   ! correction of the initial unfolding using bottom-to-top pass
      endif
   endif
   !
   !       
123 continue

enddo
Deallocate(dbz_doppler_threshold)
!!!
Deallocate(VD_unfolded_BUpass)
Deallocate(VD_unfolded_TDpass)
!
!
! check failure and redefine  folding%VD_unf_flag accordingly
! =-1 if initial correction doesn't work
! =-2 if correction using top-to-bottom filter does not work
! =-3 if correction using bottom-to-top filter does not work
!
k=0
do it=1,nt
    if (MaxVal(folding%VD_unfolded(:,it))==m999) go to 231
      k=0
      !ktr=nint(dble(count(folding%VD_unfolded(:,it)/=m999))/10.d0) ! correction fails if more than 10 % of cloudy pixels remains uncorrected
      ktr=1  !  correction falls if there is any cloud pixel that remain uncorrected
      A1:do ig=1,ng-1
        if ((folding%VD_unfolded(ig,it)/=m999) .and. (folding%VD_unfolded(ig+1,it)/=m999)) then
            diff_VD=abs(folding%VD_unfolded(ig,it)-folding%VD_unfolded(ig+1,it))    
                if (diff_VD>rad_syst%nyquist_velo(1)) k=k+1
                   if (k>ktr) then
                      folding%VD_unf_flag(it)=-folding%VD_unf_flag(it)
                      Exit A1
                   endif
         endif
      enddo A1           
      !
231 continue
enddo
!
! get folding%percentage_flag using folding%VD_unf_flag( -999 no cloud ; >=0 ok, <0 correction failed )
! percentage flag = 100 *  Sum(elevations where folding correction failed)/total of elevations containing cloud
!
k=0
k=count(folding%VD_unf_flag>-99)  ! sum of  all elevations containing cloud
if (k>0) Then
k1=count((folding%VD_unf_flag>-99) .and. (folding%VD_unf_flag<0)) ! sum of elevations where correction failed (more than 10 % cloud pixels in the ray remain uncorrected)
work1=dble(k1)/dble(k)
folding%percentage_flag(1)=work1*100.d0
endif
!
!
! redefine folding%VD_unf_flag for the official output
! =1 correction works
! =0 correction fails
where(folding%VD_unf_flag >=0) folding%VD_unf_flag=1
where((folding%VD_unf_flag >-99).and.(folding%VD_unf_flag<0)) folding%VD_unf_flag=0
!
return
end subroutine dealiasing
!!
!!
subroutine compute_cost_function_for_folding(ng,UU,VD,cf)
Use write_messages
Implicit None
!
integer, Intent(In)                :: ng
real*8,  Intent(In)                :: UU(ng)
real*8,  Intent(In)                :: VD(ng)
real*8,  Intent(Out)               :: cf
!
integer                            :: nn
real*8,Dimension(:),Allocatable    :: diff
real*8                             :: mean_diff
integer                            :: ig,inn
real*8,parameter                   :: m999=-999.d0
!
nn=0
nn=count((UU/=m999).and.(VD/=m999))
!
    if (nn<2) then
       call write_error('problem: nn<2, subroutine compute_cost_function_for_folding')
       call Exit (1)
    endif
!
Allocate(diff(nn))
diff=m999
!
inn=0
do ig=1,ng
  if ( (UU(ig)/=m999).and.(VD(ig)/=m999) ) Then
     inn=inn+1
     diff(inn)=UU(ig)-VD(ig)
  end if
enddo
!
    if ((inn/=nn).or.(MinVal(diff)==m999)) then
       call write_error('problem in diff, subroutine compute_cost_function_for_folding')
       call Exit(1)
    endif
!
mean_diff=(Sum(diff))/dble(nn)
!
cf=0.d0
do inn=1,nn
cf=cf + (diff(inn)-mean_diff)*(diff(inn)-mean_diff)
enddo
!
cf=cf/dble(nn-1)  ! variance
!
Deallocate(diff)
!
return
end subroutine compute_cost_function_for_folding
!!
subroutine final_masking(sacr_param,raw_data_per_scan,folding)
Use radar_mods
Implicit None
!
Type(sacr_parameters),Intent(In)              :: sacr_param
Type(sacr_raw_data_per_scan),Intent(InOut)    :: raw_data_per_scan
Type(sacr_folding_vars),Intent(InOut)         :: folding
!
integer                                       :: itime
integer,Dimension(:,:),Allocatable            :: mask
real*8,Parameter                              :: m999=-999.d0
!
Allocate(mask(raw_data_per_scan%ngates,raw_data_per_scan%ntimes))
mask=1
!
do itime=1,raw_data_per_scan%ntimes
  WHERE (raw_data_per_scan%DBZ(:,itime)<-100) mask(:,itime)=0
enddo
!
call maskingF(raw_data_per_scan%ngates,raw_data_per_scan%ntimes,mask)

do itime=1,raw_data_per_scan%ntimes
   WHERE (mask(:,itime)==0)
      raw_data_per_scan%DBZ(:,itime)=m999
      raw_data_per_scan%VELO(:,itime)=m999
      raw_data_per_scan%WIDTH(:,itime)=m999
      raw_data_per_scan%LDR(:,itime)=m999
      raw_data_per_scan%DBZ_rc(:,itime)=m999
      raw_data_per_scan%DBZ_cor(:,itime)=m999
      folding%VD_unfolded(:,itime)=m999
      folding%Up_az_el(:,itime)=m999
      folding%Up_az(:,itime)=m999
      folding%Up_90(:,itime)=m999
      folding%wf(:,itime)=m999
      folding%wd(:,itime)=m999
      folding%aliasn(:,itime)=int(m999)
    END WHERE
enddo
!
Deallocate(mask)
!
return
end subroutine final_masking
!!
subroutine ldr_mask(raw_data_per_scan)
Use radar_mods
Implicit None
!
Type(sacr_raw_data_per_scan),Intent(InOut)    :: raw_data_per_scan
!
integer             :: ng,nt
integer             :: ig, it
real*8, parameter   :: m999=-999.d0
real*8              :: mldr1,mldr2,dmldr,max_ldr
integer             :: ng2,igp1,igm1,nn,nnc
integer,Allocatable,Dimension(:)     :: mask

nt=raw_data_per_scan%ntimes
ng=raw_data_per_scan%ngates
!
do it=1,nt
!
!mean ldr before masking
mldr1=Sum(raw_data_per_scan%LDR(:,it))/dble(ng)
!mean ldr after masking
mldr2=m999
ng2=-1
ng2=count(raw_data_per_scan%Mask1(:,it)==1)
if (ng2<=0) go to 444
if (ng2>0) mldr2=Sum(raw_data_per_scan%LDR(:,it),MASK=raw_data_per_scan%Mask1(:,it)==1)/dble(ng2)
!
dmldr=abs(mldr1-mldr2)
!
if (dmldr<2) then
!
Allocate(mask(ng))
mask(1:ng)=raw_data_per_scan%Mask1(:,it)
max_ldr=MaxVal(raw_data_per_scan%LDR(:,it),MASK=raw_data_per_scan%Mask1(:,it)==1)

do ig=1,ng
 if (raw_data_per_scan%Mask1(ig,it)==1) then
 !
   if ((raw_data_per_scan%DBZ(ig,it)<10.d0).and.(raw_data_per_scan%LDR(ig,it)<-5.d0)) then
      igm1=MAX(1,ig-2) ; igp1=MIN(ig+2,ng); nnc=nint((dble(igp1-igm1)+0.1d0)/2.d0)
      nn=count(raw_data_per_scan%Mask1(igm1:igp1,it)==1)
      igm1=MAX(1,ig-1) ; igp1=MIN(ig+1,ng); 
      if ( (raw_data_per_scan%Mask1(igm1,it)/=1).and.(raw_data_per_scan%Mask1(igp1,it)/=1) ) then
         mask(ig)=-1
      else if (nn<=nnc) then
         mask(ig)=-1
      endif

      if (raw_data_per_scan%LDR(ig,it) >= MAX(mldr2,max_ldr-1.d0)) mask(ig)=-1
   else
   mask(ig)=-1
   endif
 endif
enddo
!
do ig=1,ng
 if(mask(ig)==-1) then
   igm1=MAX(1,ig-2) ; igp1=MIN(ig+2,ng)
   nn=count(mask(igm1:igp1)==1)
   nnc=nint((dble(igp1-igm1)+0.1d0)/2.d0)
   if ((mask(igp1)/=1).and.(mask(igm1)/=1)) then
      raw_data_per_scan%Mask1(ig,it)=0
   else if (nn<=nnc) then
      raw_data_per_scan%Mask1(ig,it)=0
   endif
 endif
enddo
!
Deallocate(mask)
!
endif ! if (dmldr<2)
!
444 continue ! if no cloud pixels in ray
enddo ! it
!
return
end subroutine ldr_mask
!!
