!! ----------------------------------------------------------------------------
!! *PROGRAM* radar_mods
!! @version sacr 1.1
!!
!! *SRC_FILE*
!! sacr/radar_mods.f90
!!
!!
!! *LAST CHANGES*
!!
!!  -May 16, 2014: A.T.  Added new variables in data structure sacr_sounding_data:
!!                       dt_sounding_scan ! [sec] time interval between sounding time and sacr scan time
!!
!!  -May 15, 2014: A.T.  Added new variables in data structure sacr_sounding_data:
!!                       soundeID         ! =1  soundings available, =0 sounding inp. files not available
!!                       msoundeID        ! =1  mergesonde data available, =0 mergesonde data not available
!!                       nt               ! number of vertical profiles in sounding/mergesonde file
!!
!!  -May 14, 2014: A.T.  Added new variables in data structure sacr_ceilo_data
!!                       ceiloID       ! =1 regular run with ceilometer data ,!=0 code executed without required ceilo inp. files
!!
!!
!!
!! *DESCRIPTION* 
!!
!! This module contains a number of subroutines for (de-)allocating, and nullifying
!! (or setting to the missed value) data types used in sacr.  These data types are
!!  also defined in this module.
!!
!!  Copyright (C) 2014 McGill Clouds Research Group (http://www.clouds.mcgill.ca/)
!!  Contact email address: pavlos.kollias@mcgill.ca, aleksandra.tatarevic@mcgill.ca, iengjo@meteo.mcgill.ca
!!
!!  This source code is free software: you can redistribute it and/or modify
!!  it under the terms of the GNU Lesser General Public License as published by
!!  the Free Software Foundation, either version 3 of the License, or
!!  any later version.
!!
!!  This program is distributed in the hope that it will be useful,
!!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!!  GNU Lesser General Public License for more details.
!!
!!  You should have received a copy of the GNU Lesser General Public License
!!  along with this program.  If not, see <http://www.gnu.org/licenses/>.
!!
!! ----------------------------------------------------------------------------
module radar_mods
implicit none
!
Type sacr_parameters
character(len=200)             :: sacr_home
character(len=6)               :: radar
character(len=3)               :: location
character(len=8)               :: date
character(len=5)               :: mode
!
character(len=150)             :: path_noise_clear_sky
integer                        :: csweep ! apply (=1) or not (=0) the correction for sweep indices
character(len=150)             :: path_processed_data ! the common path for the location of output directories/files
!
real*8                         :: ste_el_min ! [degrees] elevation value below which the second trip echo occurence is checked
real*8                         :: ste_el_max ! [degrees] elevation value over which the second trip echo occurence is checked
real*8                         :: ste_vel_thresh ! [ms^-1] treshold value of velocity standard deviation, 
                                                 ! if std > std_vel_thresh the cloud pixel is identified as the sec.trip echo 
real*8                         :: ir_ldr_thresh  ! [dB] LDR treshold in dB for the insect removal below min(first cloud base, 5 degrees izotherm),
                                                 ! insects if LDR>LDR threshoold
integer                        :: steID          ! whether or not to apply the second trip echo removal, =1 yes,=0 no
integer                        :: irID           ! whether or not to apply the insect removal, =1 yes,=0 no
integer                        :: fmID           ! whether or not to apply the final masking, =1 yes,=0 no
integer                        :: M1ID           ! whether or not to apply the mask1 (removing noise pixels using H&S74 and/or climat.noise values), =1 yes,=0 no
integer                        :: M2ID           ! whether or not to apply the ldr mask, =1 yes,=0 no
integer                        :: M3ID           ! whether or not to apply the masking using the ibox filter, =1 yes,=0 no
!
End Type sacr_parameters
!!
Type sacr_raw_data
!
integer                        :: ntimes  ! number of elevations
integer                        :: ngates  ! number of range gates
integer                        :: nsweep  ! scan number
integer                        :: nfrequency ! number of frequencies
integer                        :: one
!
integer,Dimension(:),Allocatable       :: numb_scans   ! Sweep number,1-based
integer,Dimension(:),Allocatable       :: sweep_start_ray_index  ! Index of first ray in sweep, 1-based
integer,Dimension(:),Allocatable       :: sweep_end_ray_index ! Index of last ray in sweep, 1-based
real*8 ,Dimension(:),Allocatable       :: base_time   ! seconds since 1970-1-1 0:00:00 0:00
real*8 ,Dimension(:),Allocatable       :: offset_time ! [sec] Time offset from base_time
real*8 ,Dimension(:),Allocatable       :: range ! [m]  Range to measurement volume
real*8 ,Dimension(:),Allocatable       :: azimuth ! [degrees] Azimuth angle from true east
real*8 ,Dimension(:),Allocatable       :: elevation ! [degrees] Elevation angle from horizontal
real*8 ,Dimension(:),Allocatable       :: time_s ! [sec] base_time+offset_time
real*8 ,Dimension(:,:),Allocatable     :: DBZ    ! [dBZ] Equivalent reflectivity factor
real*8 ,Dimension(:,:),Allocatable     :: VELO   ! [ms^-1] Mean Doppler velocity (negative towards instr.)
real*8 ,Dimension(:,:),Allocatable     :: SNR    ! [dB] Signal-to-noise-ratio
real*8 ,Dimension(:,:),Allocatable     :: LDR    ! [dB] Linear depolarization ratio H
real*8 ,Dimension(:,:),Allocatable     :: WIDTH  ! [ms^-1] Spectrum width
!
End Type sacr_raw_data
!
Type sacr_raw_data_per_scan
!
integer                        :: ntimes  ! number of elevations
integer                        :: ngates  ! number of range gates
integer                        :: one     ! =1
!
real*8 ,Dimension(:),Allocatable       :: elevation  ! [degrees] Elevation angle from horizontal
real*8 ,Dimension(:),Allocatable       :: azimuth    ! [degrees] Azimuth angle from true east
real*8 ,Dimension(:),Allocatable       :: time_s     !  seconds since 1970-1-1 0:00:00 0:00
real*8 ,Dimension(:),Allocatable       :: radar_time ! [hours] time since midnight in houres
real*8 ,Dimension(:),Allocatable       :: range      ! [m]  Range to measurement volume
real*8 ,Dimension(:,:),Allocatable     :: DBZ        ! [dBZ] Equivalent reflectivity factor
real*8 ,Dimension(:,:),Allocatable     :: VELO       ! [ms^-1] Mean Doppler velocity (negative towards instr.)
real*8 ,Dimension(:,:),Allocatable     :: SNR        ! [dB] Signal-to-noise-ratio
real*8 ,Dimension(:,:),Allocatable     :: LDR        ! [dB] Linear depolarization ratio H
real*8 ,Dimension(:,:),Allocatable     :: WIDTH      ! [ms^-1] Spectrum width
real*8 ,Dimension(:,:),Allocatable     :: DBZ_rc     ! [dBZ] Range corrected DBZ used for masking
real*8 ,Dimension(:,:),Allocatable     :: DBZ_cor    ! [dBZ] Reflectivity corrected for gaseous attenuation
integer,Dimension(:),Allocatable       :: fixed_angle ! [degrees] sweep fixed angle
character(len=15)                      :: time_str    ! time stamp YYYYMMDD_HHMMSS corresponding to the time at the middle of scan
integer                                :: time_str_index  ! time_s index corresponding to time_str, time_s(time_str_index)<=>time_str
integer,Dimension(:,:),Allocatable     :: Mask1     ! First level mask using noise floor (0=noise, 1=above the noise)
integer,Dimension(:,:),Allocatable     :: Mask2     ! Second level mask using noise floor (0=noise, 1=above the noise)
real*8 ,Dimension(:,:),Allocatable     :: z_distance          ! [m] distance from the ground up 
real*8 ,Dimension(:,:),Allocatable     :: x_distance          ! [m] distance away from the radar
real*8 ,Dimension(:,:),Allocatable     :: range_corrected     ! [m] range corrected for Earth's curvature 
real*8 ,Dimension(:,:),Allocatable     :: elevation_corrected ! [m] elevations corrected for Earth's curvature
!
End Type sacr_raw_data_per_scan
!
Type sacr_noise
integer                                :: nn ! number of noise data
real*8 ,Dimension(:),Allocatable       :: n_mean    ! [dB] mean clear sky climatological noise
real*8 ,Dimension(:),Allocatable       :: n_std     ! [dB] stand. deviation of clear sky climatological noise
real*8 ,Dimension(:),Allocatable       :: elevation ! [degrees] elevation anles from 1->179
End Type sacr_noise
!
Type sacr_system_constants
integer                                         :: one     ! =1
integer                                         :: two     ! =2
character(len=360)                              :: rfilename   ! input raw radar data netcdf file
character(len=360)                              :: sfilename   ! input mergesonde or sounding netcdf file
character(len=360)                              :: cfilename   ! input ceilometer netcdf file
real*8 ,Dimension(:),Allocatable                :: latitude  ! [degree_N] North latitude
real*8 ,Dimension(:),Allocatable                :: longitude ! [degree_E] Longitude
real*8 ,Dimension(:),Allocatable                :: altitude_sea_level   ! [m] Altitude above mean sea level
real*8 ,Dimension(:),Allocatable                :: altitude_from_ground ! [m] Altitude from ground
real*8 ,Dimension(:),Allocatable                :: frequency ! [GHz] Operating frequency
real*8 ,Dimension(:),Allocatable                :: nyquist_velo ! [ms^-1] Unambiguous Doppler velocity

!!
real*8                                 :: dbz_fact,velo_fact,sw_fact,snr_fact,ldr_fact ! variables unpackaging factor (X)
real*8                                 :: dbz_offset,velo_offset,sw_offset,snr_offset,ldr_offset ! variables unpackaging offset (+)
End Type sacr_system_constants
!
!
Type sacr_sounding_data
integer                                :: soundeID         ! =1  soundings available, =0  sounding inp. files not available
integer                                :: msoundeID        ! =1  mergesonde data available, =0 mergesonde data not available
real*8                                 :: dt_sounding_scan ! [sec] time interval between sounding time and sacr scan time 
integer                                :: sounde           ! number of sounding/mergesonde data in vertical
integer                                :: nt               ! number of vertical profiles in sounding/mergesonde file 
real*8 ,Dimension(:),Allocatable       :: altitude         ! [m] Altitude above mean sea level
real*8 ,Dimension(:),Allocatable       :: Temperature      ! [C] Dry Bulb Temperature
real*8 ,Dimension(:),Allocatable       :: pressure         ! [hPa] Pressure
real*8 ,Dimension(:),Allocatable       :: relative_hum     ! [%] Relative Humidity
real*8 ,Dimension(:),Allocatable       :: wf               ! [ms^-1] Wind Speed
real*8 ,Dimension(:),Allocatable       :: wd               ! [degrees] Wind Direction in mathemat. coord. system (East=0,North=90 degrees)
End Type sacr_sounding_data
!
Type sacr_ceilo_data
integer                                :: ceiloID       ! =1 regular run with ceilometer data , =0 code executed without required ceilo inp. files
integer                                :: nt            ! number of profiles
real*8 ,Dimension(:),Allocatable       :: time_s        ! time in second since 1970-1-1 0:00:00 0:00 for each profile
real*8 ,Dimension(:),Allocatable       :: first_cbh     ! [m] Lowest cloud base height detected above ground level
!integer,Dimension(:),Allocatable       :: qc_first_cbh  ! [-] quality check for first_cbh:  If ==0  a lowest cloud base height was detected, provading that detection_status has values >=1 and <=3 and qc_detection_status ==0 
!integer,Dimension(:),Allocatable       :: detection_status ! [-] number of cloud bases detected if >=1 and <=3
!integer,Dimension(:),Allocatable       :: qc_detection_status ! [-] quality check for detection_status, detection_status values are valid if qc =0
real*8                                 :: alt           ! [m] Altitude above mean sea level
End Type sacr_ceilo_data
!
Type sacr_folding_vars
integer                                       :: ntimes  ! number of elevations
integer                                       :: ngates  ! number of range gates
integer                                       :: one     ! =1
integer,Dimension(:,:),Allocatable            :: aliasn  ! number of aliasing
real*8 ,Dimension(:,:),Allocatable            :: wf      ! [ms^-1] wind force 
real*8 ,Dimension(:,:),Allocatable            :: wd      ! [degrees]  wind direction
real*8 ,Dimension(:,:),Allocatable            :: Up_90   ! [ms^-1] Wind speed from sounding projected 90 degrees to the radar scan
real*8 ,Dimension(:,:),Allocatable            :: Up_az   ! [ms^-1] Wind speed from sounding projected to the azimuths of radar scan
real*8 ,Dimension(:,:),Allocatable            :: Up_az_el  ! [ms^-1] Wind speed from sounding projected to the azimuths and elevations of radar scan
real*8 ,Dimension(:,:),Allocatable            :: VD_unfolded     ! [ms^-1] Unfolded Doppler Velocity
integer,Dimension(:),  Allocatable            :: VD_unf_flag     ! de-aliasing quality control (=0 ok, =1 correction failed)
real*8, Dimension(:),  Allocatable            :: percentage_flag ! [%] Number of rays where folding correction failed /number of rays with cloud
End Type sacr_folding_vars


Contains
!!
subroutine allocate_sacr_raw_data(str)
Implicit None
Type(sacr_raw_data),Intent(InOut)            :: str
integer                                      :: nt,ng,ns,nf,one
!
nt=str%ntimes
ng=str%ngates
ns=str%nsweep
nf=str%nfrequency
one=str%one
!
Allocate(str%numb_scans(ns))
Allocate(str%sweep_start_ray_index(ns))
Allocate(str%sweep_end_ray_index(ns))
Allocate(str%base_time(one))
Allocate(str%offset_time(nt))
Allocate(str%range(ng))
Allocate(str%azimuth(nt))
Allocate(str%elevation(nt))
Allocate(str%time_s(nt))
Allocate(str%DBZ(ng,nt))
Allocate(str%VELO(ng,nt))
Allocate(str%SNR(ng,nt))
Allocate(str%LDR(ng,nt))
Allocate(str%WIDTH(ng,nt))
!
return
end subroutine allocate_sacr_raw_data
!
subroutine deallocate_sacr_raw_data(str)
Implicit None
Type(sacr_raw_data),Intent(InOut)            :: str
!
Deallocate(str%numb_scans)
Deallocate(str%sweep_start_ray_index)
Deallocate(str%sweep_end_ray_index)
Deallocate(str%base_time)
Deallocate(str%offset_time)
Deallocate(str%range)
Deallocate(str%azimuth)
Deallocate(str%elevation)
Deallocate(str%time_s)
Deallocate(str%DBZ)
Deallocate(str%VELO)
Deallocate(str%SNR)
Deallocate(str%LDR)
Deallocate(str%WIDTH)
!
str%ntimes=0
str%ngates=0
str%nsweep=0
str%nfrequency=0
str%one=0
!
return
end subroutine deallocate_sacr_raw_data
!!
!!
subroutine initialize_sacr_raw_data(str)
Implicit None
Type(sacr_raw_data),Intent(InOut)            :: str
ReaL*8,parameter                             :: m999=-999.d0
!
str%numb_scans=int(m999)
str%sweep_start_ray_index=int(m999)
str%sweep_end_ray_index=int(m999)
str%base_time=m999
str%offset_time=m999
str%range=m999
str%azimuth=m999
str%elevation=m999
str%time_s=m999
str%DBZ=m999
str%VELO=m999
str%SNR=m999
str%LDR=m999
str%WIDTH=m999
!
return
end subroutine initialize_sacr_raw_data
!
subroutine allocate_sacr_raw_data_per_scan(str)
Implicit None
Type(sacr_raw_data_per_scan),Intent(InOut)            :: str
integer                                               :: nt,ng,one
!
nt=str%ntimes
ng=str%ngates
one=str%one
!
Allocate(str%elevation(nt))
Allocate(str%azimuth(nt))
Allocate(str%time_s(nt))
Allocate(str%radar_time(nt))
Allocate(str%range(ng))
Allocate(str%DBZ(ng,nt))
Allocate(str%DBZ_cor(ng,nt))
Allocate(str%VELO(ng,nt))
Allocate(str%SNR(ng,nt))
Allocate(str%LDR(ng,nt))
Allocate(str%WIDTH(ng,nt))
Allocate(str%DBZ_rc(ng,nt))
Allocate(str%fixed_angle(one))
Allocate(str%Mask1(ng,nt))
Allocate(str%Mask2(ng,nt))
Allocate(str%z_distance(ng,nt))
Allocate(str%x_distance(ng,nt))
Allocate(str%range_corrected(ng,nt))
Allocate(str%elevation_corrected(ng,nt))
!
return
end subroutine allocate_sacr_raw_data_per_scan
!!
subroutine deallocate_sacr_raw_data_per_scan(str)
Implicit None
Type(sacr_raw_data_per_scan),Intent(InOut)            :: str
!
Deallocate(str%elevation)
Deallocate(str%azimuth)
Deallocate(str%time_s)
Deallocate(str%radar_time)
Deallocate(str%range)
Deallocate(str%DBZ)
Deallocate(str%DBZ_cor)
Deallocate(str%VELO)
Deallocate(str%SNR)
Deallocate(str%LDR)
Deallocate(str%WIDTH)
Deallocate(str%DBZ_rc)
Deallocate(str%fixed_angle)
Deallocate(str%Mask1)
Deallocate(str%Mask2)
Deallocate(str%z_distance)
Deallocate(str%x_distance)
Deallocate(str%range_corrected)
Deallocate(str%elevation_corrected)
!
str%ntimes=0
str%ngates=0
str%one=0
!
return
end subroutine deallocate_sacr_raw_data_per_scan
!
subroutine initialize_sacr_raw_data_per_scan(str)
Implicit None
Type(sacr_raw_data_per_scan),Intent(InOut)            :: str
ReaL*8,parameter                                      :: m999=-999.d0
!
str%elevation=m999
str%azimuth=m999
str%time_s=m999
str%radar_time=m999
str%range=m999
str%DBZ=m999
str%DBZ_cor=m999
str%VELO=m999
str%SNR=m999
str%LDR=m999
str%WIDTH=m999
str%DBZ_rc=m999
str%fixed_angle=int(m999)
str%time_str='NULL'
str%Mask1=int(m999)
str%Mask2=int(m999)
str%z_distance=m999
str%x_distance=m999
str%range_corrected=m999
str%elevation_corrected=m999
!
return
end subroutine initialize_sacr_raw_data_per_scan
!
subroutine allocate_sacr_noise_data(str)
Implicit None
Type(sacr_noise),Intent(InOut)            :: str
integer                                   :: nn
!
nn=str%nn
!
Allocate(str%n_mean(nn))
Allocate(str%n_std(nn))
Allocate(str%elevation(nn))
!
return
end subroutine allocate_sacr_noise_data
!
subroutine deallocate_sacr_noise_data(str)
Implicit None
Type(sacr_noise),Intent(InOut)            :: str
!
Deallocate(str%n_mean)
Deallocate(str%n_std)
Deallocate(str%elevation)
!
str%nn=0
return
end subroutine deallocate_sacr_noise_data
!
subroutine initialize_sacr_noise_data(str)
Implicit None
Type(sacr_noise),Intent(InOut)            :: str
Real*8,Parameter                          :: m999=-999.d0
!
str%n_mean    = m999
str%n_std     = m999
str%elevation = m999
!
return
end subroutine initialize_sacr_noise_data
!
subroutine allocate_sacr_system_constants(str)
Implicit None
Type(sacr_system_constants),Intent(InOut)     :: str
integer                                       :: nn
!
nn=str%one
!
Allocate(str%latitude(nn))
Allocate(str%longitude(nn))
Allocate(str%altitude_sea_level(nn))
Allocate(str%altitude_from_ground(nn))
Allocate(str%frequency(nn))
Allocate(str%nyquist_velo(nn))
!
return
end subroutine allocate_sacr_system_constants
!
subroutine deallocate_sacr_system_constants(str)
Implicit None
Type(sacr_system_constants),Intent(InOut)     :: str
!
Deallocate(str%latitude)
Deallocate(str%longitude)
Deallocate(str%altitude_sea_level)
Deallocate(str%altitude_from_ground)
Deallocate(str%frequency)
Deallocate(str%nyquist_velo)
!
return
end subroutine deallocate_sacr_system_constants
!
subroutine initialize_sacr_system_constants(str)
Implicit None
Type(sacr_system_constants),Intent(InOut)            :: str
Real*8,Parameter                                     :: m999=-999.d0
!
str%rfilename=''
str%sfilename=''
str%cfilename=''
str%latitude=m999
str%longitude=m999
str%altitude_sea_level=m999
str%altitude_from_ground=m999
str%frequency=m999
str%nyquist_velo=m999
!
str%dbz_fact=m999
str%velo_fact=m999
str%sw_fact=m999
str%snr_fact=m999
str%ldr_fact=m999
str%dbz_offset=m999
str%velo_offset=m999
str%sw_offset=m999
str%snr_offset=m999
str%ldr_offset=m999
!
return
end subroutine initialize_sacr_system_constants
!
subroutine allocate_sacr_sounding_data(str)
Implicit None
Type(sacr_sounding_data),Intent(InOut)            :: str
integer                                           :: ns
!
ns=str%sounde
Allocate(str%altitude(ns))
Allocate(str%Temperature(ns))
Allocate(str%pressure(ns))
Allocate(str%relative_hum(ns))
Allocate(str%wf(ns))
Allocate(str%wd(ns))
!
return
end subroutine allocate_sacr_sounding_data
!
subroutine deallocate_sacr_sounding_data(str)
Implicit None
Type(sacr_sounding_data),Intent(InOut)            :: str
!
Deallocate(str%altitude)
Deallocate(str%Temperature)
Deallocate(str%pressure)
Deallocate(str%relative_hum)
Deallocate(str%wf)
Deallocate(str%wd)
!
str%sounde=0
str%nt=0
!
return
end subroutine deallocate_sacr_sounding_data
!
subroutine initialize_sacr_sounding_data(str)
Implicit None
Type(sacr_sounding_data),Intent(InOut)       :: str
ReaL*8,parameter                             :: m999=-999.d0
!
str%altitude=m999
str%Temperature=m999
str%pressure=m999
str%relative_hum=m999
str%wf=m999
str%wd=m999
!
str%dt_sounding_scan=m999
!
return
end subroutine initialize_sacr_sounding_data
!
subroutine allocate_sacr_ceilo_data(str)
Implicit None
Type(sacr_ceilo_data),Intent(InOut)            :: str
integer                                        :: nt
!
nt=str%nt
Allocate(str%time_s(nt))
Allocate(str%first_cbh(nt))
!
return
end subroutine allocate_sacr_ceilo_data
!
subroutine deallocate_sacr_ceilo_data(str)
Implicit None
Type(sacr_ceilo_data),Intent(InOut)            :: str
!
Deallocate(str%time_s)
Deallocate(str%first_cbh)
!
str%nt=0
!
return
end subroutine deallocate_sacr_ceilo_data
!
subroutine initialize_sacr_ceilo_data(str)
Type(sacr_ceilo_data),Intent(InOut)            :: str
ReaL*8,parameter                              :: m999=-999.d0
!
str%time_s=m999
str%first_cbh=m999
!
return
end subroutine initialize_sacr_ceilo_data
!!
!!
subroutine allocate_sacr_folding_vars(str)
Implicit None
Type(sacr_folding_vars),Intent(InOut)            :: str
integer                                          :: ng,nt,one
!
nt=str%ntimes
ng=str%ngates
one=str%one
!
Allocate(str%aliasn(ng,nt))
Allocate(str%wf(ng,nt))
Allocate(str%wd(ng,nt))
Allocate(str%Up_90(ng,nt))
Allocate(str%Up_az(ng,nt))
Allocate(str%Up_az_el(ng,nt))
Allocate(str%VD_unfolded(ng,nt))
Allocate(str%VD_unf_flag(nt))
Allocate(str%percentage_flag(one))
!
return
end subroutine allocate_sacr_folding_vars
!
subroutine deallocate_sacr_folding_vars(str)
Implicit None
Type(sacr_folding_vars),Intent(InOut)            :: str
!
Deallocate(str%aliasn)
Deallocate(str%wf)
Deallocate(str%wd)
Deallocate(str%Up_90)
Deallocate(str%Up_az)
Deallocate(str%Up_az_el)
Deallocate(str%VD_unfolded)
Deallocate(str%VD_unf_flag)
Deallocate(str%percentage_flag)
!
str%ntimes=0
str%ngates=0
str%one=0
!
return
end subroutine deallocate_sacr_folding_vars
!
subroutine initialize_sacr_folding_vars(str)
Type(sacr_folding_vars),Intent(InOut)            :: str
ReaL*8,parameter                                 :: m999=-999.d0
!
str%aliasn=int(m999)
str%wf=m999
str%wd=m999
str%Up_90=m999
str%Up_az=m999
str%Up_az_el=m999
str%VD_unfolded=m999
str%VD_unf_flag=int(m999)
str%percentage_flag=m999
!
return
end subroutine initialize_sacr_folding_vars
!
end module radar_mods
