!!-----------------------------------------------------------------------------------------------------------------
!! *PROGRAM* : sacr
!! @version sacr 1.1
!!
!! *SRC_FILE*
!! /sacr/sacr.f90
!!
!!
!! *LAST CHANGES* 
!! 
!!  -May 23, 2014: A.T. Added generic noise file as an input for sites without them
!!
!!  -May 16, 2014: A.T. Added new output variable -time interval between the scan time 
!!                      and the closest sounding  
!!
!!  -May 15, 2014: A.T. Introduced merged soundings as an input:
!!                      - mergedsonde_data_file list file is introduced as an input file
!!                      - code looks first if  the merged data are available, if not it 
!!                        checks for soundings,and if soundings are not available, it
!!                        proceeds to run without this input
!!                      - Also added option in configuration enabling user to execute 
!!                        the code exclusively with merged soundings or soundings.  
!!
!!  -May 14, 2014: A.T. Code optimised to run even if there is no input ceilometer or 
!!                      sounding data. Also added option in configuration to voluntarily 
!!                      exclude any of them.
!! 
!!  -Apr 17, 2014: A.T. Corrected bug in IF condition when calling CheckElFlippingAndRepeating
!!                      for ppi and vpo mode (.or. replaced by .and.) 
!!
!!  
!!
!! *DESCRIPTION* 
!!
!! This model processes calibrated SACR measurements and outputs quality-controlled post-procesing measurements after:
!! 1. the construction of hydrometeor mask and removal of not significant radar returns
!! 2. the attenution correction for gasses using the nearest-in-time atmospheric sounding
!! 3. the corrrection for the velocity folding that makes use of a first guess for the true Doppler velocity using 
!!    horizontal wind measurements from the nearest sounding
!! 4. the insect removal 
!! 5. the removal of the second trip echo
!!
!!
!!
!! *INPUT*
!!
!! the input files and configuration parameters are specified in the run_sacr.sh
!! For additional information, please see SACR(m) User Guide.
!!
!! *OUTPUT* 
!!
!! A set of  netcdf output files is generated, each corresponding to one specific 
!! observational scan  
!!
!!
!!  Copyright (C) 2014 McGill Clouds Research Group (http://www.clouds.mcgill.ca/)
!!  Contact email address: pavlos.kollias@mcgill.ca, aleksandra.tatarevic@mcgill.ca, iengjo@meteo.mcgill.ca
!!
!!  This source code is free software: you can redistribute it and/or modify
!!  it under the terms of the GNU Lesser General Public License as published by
!!  the Free Software Foundation, either version 3 of the License, or
!!  any later version.
!!
!!  This program is distributed in the hope that it will be useful,
!!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!!  GNU Lesser General Public License for more details.
!!
!!  You should have received a copy of the GNU Lesser General Public License
!!  along with this program.  If not, see <http://www.gnu.org/licenses/>.
!!
!!----------------------------------------------------------------------------------------------------------------- 
program sacr
Use radar_mods
Use write_messages
!
Implicit None
!
Type(sacr_parameters)                  :: sacr_param          ! structure for env. variables
Type(sacr_raw_data)                    :: raw_data            ! raw sacr data structure
Type(sacr_raw_data_per_scan)           :: raw_data_per_scan   ! raw sacr data per scan structure
Type(sacr_noise)                       :: noise               ! structure for clear sky noise 
Type(sacr_system_constants)            :: rad_syst            ! structure for the radar system invariables  
Type(sacr_sounding_data)               :: sounding            ! structure for sounding data
Type(sacr_ceilo_data)                  :: ceilo_data          ! structure for ceilometer data
Type(sacr_folding_vars)                :: folding             ! per scan structure containing the variables important for Dopler unfolding procedure
!
! variables per scan
integer                                          :: it_s,it_e ! first and last index of elevation array, from lowest to highest & repeated elevations exluded
integer                                          :: ntimes    ! number of elements in elevation array, from lowest to highest & repeated elevations exluded
integer                                          :: flipID    ! =0 raw elevations from lowest to highest
                                                              ! =1 flipping is required as raw elevations go from highest to lowest
                                                              ! =-1 disregard writting the output file as 1.raw elevations are identical, or
                                                              !                                           2. span of valid elevations<20 degrees

character(len=356)                     :: OutFileName ! name of output file
!
! raw input files
integer                                          :: ifile,iscan
integer                                          :: nfiles
character(len=356),Dimension(:),Allocatable      :: raw_data_file
! sounding input files
integer                                          :: nsoundings ! number of soundings associated to the site and date of interest 
character(len=356),Dimension(:),Allocatable      :: sounding_data_file ! names of sounding files  associated to the site and date of interest
integer                                          :: isounde  ! index of sounding_data_file closest to the scanning time
real*8                                           :: dt_sounding_scan ! [sec] time interval between sounding time and sacr scan time
!
character(len=356)                               :: mergedsonde_data_file ! name of mergedsonde file associated to the site and date of interest
character(len=356)                               :: ceilometer_data_file ! name of ceilometer file associated to the site and date of interest
!
integer                                          :: NoiseFileID ! =0 generic noise file, =1 sacr_aux/ noise file 
!
! misc
character(len=356)                     :: InpFile ! input file
character(len=3)                       :: set,block,fixed_angle_string
character(len=3)                       :: location
character(len=6)                       :: radar
character(len=6)                       :: wstr
character(len=9)                       :: FixAngleName
integer                                :: iset1,iset2
integer                                :: iblock1,iblock2
integer                                :: status 
integer,parameter                      :: id=1
integer,parameter                      :: idf=31
!!
!!
!-----------------------------------------
! GET  environment variables
!-----------------------------------------
! 
call get_env_vars(sacr_param)
!
!-----------------------------------------
! define the string that appears in the name of output files
FixAngleName='_azimuth_'
If (Trim(sacr_param%mode)=='ppimo') FixAngleName='_elevati_'
!
!----------------------------------------------------------------------------------
! get the number and names of raw data files
InpFile=Trim(sacr_param%location)//'_'//Trim(sacr_param%radar)//'_'//Trim(sacr_param%mode)//'_'//Trim(sacr_param%date)//'.list'
InpFile=Trim(sacr_param%sacr_home)//'path_raw_data_'//Trim(InpFile)
!
call CountLinesOfFile(InpFile,nfiles)
Allocate(raw_data_file(nfiles))
open(unit=id,file=InpFile)
write(*,*) ' '
write(*,*) '-------------------------------------------------- '
do ifile=1,nfiles
read(id,*) raw_data_file(ifile)
enddo
close(id)
!
!----------------------------------------------------------------------------------
! get the number and names of sounding data files
InpFile=Trim(sacr_param%location)//'_'//Trim(sacr_param%radar)//'_'//Trim(sacr_param%mode)//'_'//Trim(sacr_param%date)//'.list'
InpFile=Trim(sacr_param%sacr_home)//'path_sounding_data_'//Trim(InpFile)
call CountLinesOfFile(InpFile,nsoundings)
Allocate(sounding_data_file(nsoundings))
open(unit=id,file=InpFile)
do ifile=1, nsoundings
read(id,*) sounding_data_file(ifile)
enddo
close(id)
!
! check if the code is to be executed without sounding data
sounding%soundeID=1 ! regular run 
if ((nsoundings==1) .and. (sounding_data_file(1)=="NULL")) sounding%soundeID=0 
!
!----------------------------------------------------------------------------------
! get the name of mergedsonde data file
InpFile=Trim(sacr_param%location)//'_'//Trim(sacr_param%radar)//'_'//Trim(sacr_param%mode)//'_'//Trim(sacr_param%date)//'.list'
InpFile=Trim(sacr_param%sacr_home)//'path_mergedsonde_data_'//Trim(InpFile)
open(unit=id,file=InpFile)
read(id,*) mergedsonde_data_file
close(id)
!
! check if the code is to be executed without mergedsonde data
sounding%msoundeID=1 ! regular run
if (mergedsonde_data_file=="NULL") sounding%msoundeID=0
!----------------------------------------------------------------------------------
! CEILOMETER DATA
! get the name of Ceilometer data file
InpFile=Trim(sacr_param%location)//'_'//Trim(sacr_param%radar)//'_'//Trim(sacr_param%mode)//'_'//Trim(sacr_param%date)//'.list'
InpFile=Trim(sacr_param%sacr_home)//'path_ceilometer_data_'//Trim(InpFile)
open(unit=id,file=InpFile)
read(id,*) ceilometer_data_file
close(id)
!
! check if the code is to be executed without ceilometer data
ceilo_data%ceiloID=1 ! regular run
if (ceilometer_data_file=="NULL") ceilo_data%ceiloID=0
!
!
If (ceilo_data%ceiloID==1) Then ! regular run with ceilo input data
! read the Ceilometer data file
InpFile=Trim(ceilometer_data_file)
call ReadInpNCDF_dim_ceilo_data(InpFile,ceilo_data,status)
!
call allocate_sacr_ceilo_data(ceilo_data)
call initialize_sacr_ceilo_data(ceilo_data)
!
call ReadInpNCDF_var_ceilo_data(InpFile,ceilo_data,status)
!
Else ! execute code without ceilo input data
ceilo_data%nt=1
call allocate_sacr_ceilo_data(ceilo_data)
call initialize_sacr_ceilo_data(ceilo_data)
!
endif
!----------------------------------------------------------------------------------
!----------------------------------------------------------------------------------
! get the noise file
call convert_string_to_upper_or_lower_case(0,Trim(sacr_param%location),location)
call convert_string_to_upper_or_lower_case(0,Trim(sacr_param%radar),radar)
!
NoiseFileID=0 ! generic noise file
!
! sacr_aux noise files
If ((Trim(location)=='nsa').or.(Trim(location)=='pvc').or.(Trim(location)=='sgp')) Then
    If ((Trim(radar)=='kasacr').or.(Trim(radar)=='wbsacr')) NoiseFileID=1 
EndIf
If (Trim(location)=='tmp') Then
    If ((Trim(radar)=='kasacr').or.(Trim(radar)=='xasacr')) NoiseFileID=1
Endif
! 
If (NoiseFileID==0) Then
    InpFile=Trim(sacr_param%path_noise_clear_sky)//"noise_generic.dat"
Else
    InpFile=Trim(sacr_param%path_noise_clear_sky)//Trim(location)//&
                               "_noise_clear_sky_"//Trim(radar)//".dat"
EndIf
!
call CountLinesOfFile(Trim(InpFile),noise%nn)
     !
     if (noise%nn<=0) then
        call write_error('problem when reading the noise file')
        call Exit (1)
     endif
     !
call allocate_sacr_noise_data(noise)
call initialize_sacr_noise_data(noise)
call read_sacr_noise_data(InpFile,noise)
!!
!!
!!-------------------------------------------------------------------------------------------------------
InpFile=Trim(sacr_param%location)//'_'//Trim(sacr_param%radar)//'_'//Trim(sacr_param%mode)//'_'//Trim(sacr_param%date)//'.list'
InpFile=Trim(sacr_param%sacr_home)//'processed_output_files_'//Trim(InpFile)
open(unit=idf,file=InpFile)
!!
!!-------------------------------------------------------------------------------------------------------
! loop over the raw data input files
iset1=1 ; iset2=nfiles 
do ifile=iset1,iset2
!
call  write_prog_status('Processing files: SET= ',ifile,nfiles)
!
!define set
call get_3ch_string(ifile,set)
! read the file 
InpFile=raw_data_file(ifile)
call ReadInpNCDF_dim_raw_data(InpFile,raw_data,status)
!
call allocate_sacr_raw_data(raw_data)
call initialize_sacr_raw_data(raw_data)
!
rad_syst%one=raw_data%one
call allocate_sacr_system_constants(rad_syst)
call initialize_sacr_system_constants(rad_syst)
!
call get_filename(InpFile,rad_syst%rfilename)
call get_filename(Trim(ceilometer_data_file),rad_syst%cfilename)
!
call ReadInpNCDF_var_raw_data(InpFile,raw_data,rad_syst,sacr_param,status)
!
!!------------------------------------------------------------------------
!!------------------------------------------------------------------------
iblock1=1 ; iblock2=raw_data%nsweep
do iscan=iblock1,iblock2
!
call  write_prog_status('Processing files: BLOCK= ',iscan,raw_data%nsweep)
!
! define block
call get_3ch_string(iscan,block)
!
! Extract the data for the block
!
flipID=0 ! -  correction for flipping not required
it_s=raw_data%sweep_start_ray_index(iscan) ! the first elevation index
it_e=raw_data%sweep_end_ray_index(iscan)   ! the last elevation index
ntimes=it_e-it_s+1                         ! the total number of elevations
!---------------
if ((Trim(sacr_param%mode)/='vptmo') .and. (Trim(sacr_param%mode)/='ppimo')) &
    call CheckElFlippingAndRepeating(trim(sacr_param%mode),iscan,raw_data,ntimes,it_s,it_e,flipID)
!
if (flipID==-1) go to 123
! define dimensions
raw_data_per_scan%ntimes=ntimes
raw_data_per_scan%ngates=raw_data%ngates
raw_data_per_scan%one=raw_data%one
!
call allocate_sacr_raw_data_per_scan(raw_data_per_scan)
call initialize_sacr_raw_data_per_scan(raw_data_per_scan)
! extract the scan corrected for flipping and repeated elevations 
call ExtractDataPerScan(iscan,it_s,it_e,flipID,sacr_param%mode,raw_data,raw_data_per_scan)
!
call get_3ch_string(raw_data_per_scan%fixed_angle(1),fixed_angle_string)  
!
call masking(raw_data_per_scan,noise,sacr_param)
!
! Correction for the curvature of the earth
call curvature_correction(raw_data_per_scan)
!
!-----------------------------------------------------------------------------------------------
! soundings
!
if (sounding%msoundeID==1) then ! mergeddsonde data available
!
call get_filename(Trim(mergedsonde_data_file),rad_syst%sfilename)
call ReadInpNCDF_dim_mergedsonde_data(Trim(mergedsonde_data_file),sounding,status)
call allocate_sacr_sounding_data(sounding)
call initialize_sacr_sounding_data(sounding)
call ReadInpNCDF_var_mergedsonde_data(Trim(mergedsonde_data_file),raw_data_per_scan%time_str,sounding,status)
!
endif

if (sounding%msoundeID==0) then ! if mergedsonde data not available
 if (sounding%soundeID==1) then !  if sounding data available
    ! Identify the closest sounding to the scan
    call GetSoundingID(nsoundings,sounding_data_file,raw_data_per_scan%time_str,isounde,dt_sounding_scan)
    ! Read sounding data
    call get_filename(Trim(sounding_data_file(isounde)),rad_syst%sfilename)
    call ReadInpNCDF_dim_sounding_data(Trim(sounding_data_file(isounde)),sounding,status)
    call allocate_sacr_sounding_data(sounding)
    call initialize_sacr_sounding_data(sounding)
    call ReadInpNCDF_var_sounding_data(Trim(sounding_data_file(isounde)),sounding,status)
    sounding%dt_sounding_scan=dt_sounding_scan
    !
  else ! if sounding%soundeID==0 ! if sounding data not available
    rad_syst%sfilename="NULL"
    sounding%sounde=1
    sounding%nt=1
    call allocate_sacr_sounding_data(sounding)
    call initialize_sacr_sounding_data(sounding)
  endif
endif
!!
!-----------------------------------------------------------------------------------------------
! Remove the insects
    If ((ceilo_data%ceiloID==0).and.(sounding%soundeID==0).and.(sounding%msoundeID==0)) Then
       go to 134
    Else
       IF (sacr_param%irID==1) call InsectRemoval(ceilo_data,sounding,rad_syst,sacr_param,raw_data_per_scan)
    EndIf 
134 continue
!!
!-----------------------------------------------------------------------------------------------
! Second Trip Echo
   IF (sacr_param%steID==1) Then
      if (Trim(sacr_param%mode)/='vptmo') call RemoveSecondTripEcho(sacr_param,raw_data_per_scan)
   EndIF
!-----------------------------------------------------------------------------------------------
! Correction for attenuation due to gases
IF ((sounding%msoundeID==1).or.(sounding%soundeID==1)) Then ! if mergedsonde or sounding present
call EstimateGaseousAttenuation(rad_syst%frequency(1),sounding,rad_syst,raw_data_per_scan)
EndIF
!
!-----------------------------------------------------------------------------------------------
! Doppler velocity unfolding
folding%ngates=raw_data_per_scan%ngates
folding%ntimes=raw_data_per_scan%ntimes
folding%one=raw_data_per_scan%one
!
call allocate_sacr_folding_vars(folding)
call initialize_sacr_folding_vars(folding)
!
IF ((sounding%msoundeID==1) .or. (sounding%soundeID==1))  Then ! if mergedsone or sounding present
call dealiasing(rad_syst,sounding,raw_data_per_scan,folding)
EndIF
!!
!-----------------------------------------------------------------------------------------------
! Final masking
IF (sacr_param%fmID==1) Then
call final_masking(sacr_param,raw_data_per_scan,folding)
EndIF
!write(*,*) 'dbz',real(MinVal(raw_data_per_scan%DBZ,MASK=raw_data_per_scan%DBZ/=-999.d0)),real(MaxVal(raw_data_per_scan%DBZ))
!
!
!-----------------------------------------------------------------------------------------------
! write output file
Call GetOutFileDir(sacr_param,OutFileName)

OutFileName=Trim(Adjustl(OutFileName))//Trim(radar)//'_'//Trim(sacr_param%mode)//'_mode_set_'//Trim(set)//'_block_'//Trim(block)//&
            Trim(Adjustl(FixAngleName))//Trim(fixed_angle_string)//'_'//Trim(raw_data_per_scan%time_str)//'.nc'
!
write(idf,'(a)') Trim(OutFileName)
!
call write_ncdf_output(OutFileName,raw_data_per_scan,folding,rad_syst,sacr_param,sounding,ceilo_data,status)
!-----------------------------------------------------------------------------------------------
! deallocate structures
!
call deallocate_sacr_raw_data_per_scan(raw_data_per_scan)
call deallocate_sacr_sounding_data(sounding)
call deallocate_sacr_folding_vars(folding)
!
123 continue ! 
!
enddo ! iscan

!!------------------------------------------------------------------------
!!------------------------------------------------------------------------
call deallocate_sacr_system_constants(rad_syst)
call deallocate_sacr_raw_data(raw_data)
write(idf,*) '  '
write(*,*) '-------------------------------------------------- '
!
enddo ! ifile
close(idf)
call deallocate_sacr_noise_data(noise)
call deallocate_sacr_ceilo_data(ceilo_data)
Deallocate (sounding_data_file)
Deallocate(raw_data_file)
!!-------------------------------------------------------------------------------------------------------
!!-------------------------------------------------------------------------------------------------------
end program sacr
